// INOVASIS.h
#ifndef _INOVASIS_H
#define _INOVASIS_H

#define INOVASIS_VERSION "1.4.0"

#ifdef  __cplusplus
extern "C" {
#endif

#include <dsp.h>
#include <arpa/inet.h>
#include <sys/socket.h>

#ifndef DLL_EXPORT
#define DLL_EXPORT extern
#endif

/** \mainpage i.Nova Technologies Smart Imaging System SDK
*
* \section Introduction
*
* <p>The i.Nova Technologies Smart Imaging System SDK can help you to use the internal functionalities of the SIS Box or camera,</p>
* <p>The SIS Box can be extended by adding peripherals or modules, it has superior processing power and connectivity.</p>
*
* This SDK contains these modules:
* <ul>
* <li>The SIS API which contains a set of functions expecially for the SIS Box and cameras.</li>
* <li>The DSP software library that can help the developer to furher process the streams or pictures captured.</li>
* <li>RTKLIB: A library for GNSS Positioning and realtime kinematics with high precision</li>
* <li>openVLBI: a free, open source set of applications for very long baseline radio interferometry.</li>
* <li>A XML parser and writer.</li>
* <li>A base64 codec.</li>
* </ul>
*
* \author i.Nova Technologies
* \author Ilia Platone <ilia.platone@inovatech-space.com>
*/

/**
 * \defgroup SISAPI SIS API - Functions to control and manage the SIS Box
*/

/*@{*/

/**
 * \defgroup SISAPI_DEFINES Defines used in SIS SDK
*/
/*@{*/

/** Full Resolution */
#define RESOLUTION_FULL 0
/** ROI resolution */
#define RESOLUTION_ROI 1
/** BIN 2x2 resolution */
#define RESOLUTION_BIN 2

/** Low Frame speed, 12bit depht, for long exposures */
#define FRAME_SPEED_LOW 0
/** Normal frame speed, 12bit depht, at higher frame rates */
#define FRAME_SPEED_NORMAL 1
/** Maximum frame speed, 8bit depht, at full FPS */
#define FRAME_SPEED_HIGH 2

/** No sensor at all */
#define ID_NONE -1

/** SONY IMX225LQR */
#define IMX225LQR  1
/** SONY IMX290LRJ */
#define IMX290LRJ  2
/** SONY IMX185LQJ */
#define IMX185LQJ  3
/** SONY IMX385LQR */
#define IMX385LQR  4
/** SONY IMX294CJK */
#define IMX294CJK  5
/** SONY IMX287LLR */
#define IMX287LLR  6
/** SONY IMX297LLR */
#define IMX297LLR  7
/** SONY IMX297LLRX2 */
#define IMX297LLRX2  8
/** SONY IMX297LLRX4 */
#define IMX297LLRX4  9
/** SONY IMX290LQJ */
#define IMX290LQR 20


/** no direction, for ST4 */
#define DIR_NONE 0x0F
/** North direction, for ST4 */
#define NORTH 0x0D
/** West direction, for ST4 */
#define WEST 0x07
/** South direction, for ST4 */
#define SOUTH 0x0B
/** East direction, for ST4 */
#define EAST 0x0E


/** reinitialize the module, calls probe() */
#define CSI_SUBDEV_INIT_FULL 0x01
/** reinitialize the module */
#define CSI_SUBDEV_INIT_SIMP 0x02
/** toggles on the reset line */
#define CSI_SUBDEV_RST_ON 0x03
/** toggles off the reset line */
#define CSI_SUBDEV_RST_OFF 0x04
#define CSI_SUBDEV_RST_PUL 0x05
/** toggles on the standby line */
#define CSI_SUBDEV_STBY_ON 0x06
/** toggles off the standby line */
#define CSI_SUBDEV_STBY_OFF 0x07
/** powers on the module, toggles off the standby line */
#define CSI_SUBDEV_PWR_ON 0x08
/** powers off the module, toggles on the standby line */
#define CSI_SUBDEV_PWR_OFF 0x09

/** Exposure time below 1ms */
#define SUBMS_EXPTIME 0

/** button codes from IR receiver */
#define BUTTON_0 0xe91
#define BUTTON_1 0xf30c
#define BUTTON_2 0xe718
#define	BUTTON_3 0xa15e
#define BUTTON_4 0xf708
#define BUTTON_5 0xe31c
#define BUTTON_6 0xa55a
#define BUTTON_7 0xbd42
#define BUTTON_8 0xad52
#define BUTTON_9 0xb54a
#define NODATA 0xffff
#define IR_ERR -1

/*@}*/
/**
 * \defgroup SISAPI_TYPES Types used in SIS SDK
*/
/*@{*/

/** Operational resolutions */
typedef int inova_resolution_t;

/** Readout frame speeds */
typedef int inova_frame_speed_t;

/** Sensor types code ids */
typedef int inova_sensor_id_t;

/** mount directions that can pass to iNovaSIS_SendST4 */
typedef int inova_st4_direction_t;

/** commands that can pass to iNovaSIS_PowerCmd */
typedef int inova_pwr_cmd_t;

/** flags that can pass to iNovaSIS_SetFlag/iNovaSIS_GetFlag */
typedef int inova_sis_flag_t;

/** button codes from IR receiver */
typedef int inova_ir_data_t;

/** the i2c instance handle */
typedef int inova_i2c_handle_t;

/** the ir instance handle */
typedef int inova_ir_handle_t;

/** the socket instance handle */
typedef int inova_socket_handle_t;

/** the client interface */
typedef struct _iNovaSIS_Client {
	struct sockaddr addr;
	socklen_t len;
	int index; 
	size_t max_clients;
	void *server;
	inova_socket_handle_t fd;
} iNovaSIS_Client;

/** the network callback when connecting to clients */
typedef void * (inova_network_callback_t)(iNovaSIS_Client *);

/** the client interface */
typedef struct _iNovaSIS_Server {
	struct sockaddr_in addr;
	inova_socket_handle_t fd;
	iNovaSIS_Client **clients;
	unsigned long connected_mask;
	unsigned int max_clients;
	unsigned int connected_clients;
	inova_network_callback_t *send_callback;
	inova_network_callback_t *recv_callback;
} iNovaSIS_Server;


/*@}*/
/**
 * \defgroup SISAPI_BOX Functions to get or set informations of the SIS Box
*/
/*@{*/

/** \brief Current SIS version.
 *  @return the Current i.NovaSIS SIS version.
 */
DLL_EXPORT inline const char* iNovaSIS_Version() { return INOVASIS_VERSION; }

/** \brief Returns the sensor board informations.
 *  @return informations about the sensor module
*/
DLL_EXPORT char * iNovaSIS_BoardInfo();
/** \brief Returns the sensor board chip model name.
 *  @return the chip model.
 */
DLL_EXPORT char * iNovaSIS_GetName();
/** \brief Returns the serial number of the SIS Box.
 *  \param data optional data to be written.
 *  @return the serial number.
 */
DLL_EXPORT const char * iNovaSIS_GetBoxSerial(const char *data);
/** \brief Returns the serial number of the additional module.
 *  \param data optional data to be written.
 *  @return the serial number.
 */
DLL_EXPORT const char * iNovaSIS_GetBoxSensor(const char *data);
/** \brief Executes a system command and returns its output and lenght.
*  \param cmd the system command to execute.
*  \param ret the command output.
 *  @return the lenght in characters of the command output.
 */
DLL_EXPORT int iNovaSIS_SystemCmd(const char *cmd, char *ret);
/*@}*/
/**
 * \defgroup SISAPI_CAMERA Functions to get or set informations of the SIS Camera
*/
/*@{*/

/** \brief Sets the chosen flag to 1.
 *  @return the current context.
 *  \param x the flag index.
 */
DLL_EXPORT void iNovaSIS_SetFlag(inova_sis_flag_t x);
/** \brief Returns the current status of the chosen flag.
 *  @return 1 if the flag is set, 0 if the flag is clear...
 */
DLL_EXPORT int iNovaSIS_GetFlag();
/** \brief Indicates if any SIS sensor module is attached.
 *  @return the number of modules connected...
 */
DLL_EXPORT int iNovaSIS_MaxCamera();
/** \brief Closes the SIS module.
 */
DLL_EXPORT void iNovaSIS_CloseCamera();
/** \brief Opens the SIS module and gathers all informations about it.
 *  \param n the camera sensor index.
 *  @return The serial number of the attached module...
 */
DLL_EXPORT char * iNovaSIS_OpenCamera(int n);
/** \brief Gets the sensor model name string.
 *  @return the sensor model name...
 */
DLL_EXPORT char * iNovaSIS_GetSensorName();
/** \brief Gets the sensor module ID
 *  @return the sensor ID..
 */
DLL_EXPORT inova_sensor_id_t iNovaSIS_SensorID();
/** \brief Controls the power state of the SIS Module.
 *  \param cmd the command to send to the sensor module.
 */
DLL_EXPORT void iNovaSIS_PowerCmd(inova_pwr_cmd_t cmd);
/** \brief turns on the power of the SIS Module.
 */
DLL_EXPORT void iNovaSIS_PowerOn();
/** \brief turns off the power of the SIS Module.
 */
DLL_EXPORT void iNovaSIS_PowerOff();
/** \brief Obtains the stream array size in sample units.
 *  @return the size of array..
 */
DLL_EXPORT int iNovaSIS_GetArraySize();
/** \brief Initializes the SIS module.
 *  \param resolution the resolution type.
 *  \param n the size of the circular buffer.
 *  @return 1 if success, 0 if  failure.
 */
DLL_EXPORT int iNovaSIS_InitCamera(inova_resolution_t resolution, int n);
/** \brief Starts streaming from the sensor module.
 */
DLL_EXPORT void iNovaSIS_OpenVideo();
/** \brief Stops streaming from the sensor module.
 */
DLL_EXPORT void iNovaSIS_CloseVideo();
/** \brief Grabs a frame from the module, if any.
 *  @return NULL if not ready, otherwise the image captured.
 */
DLL_EXPORT void* iNovaSIS_GrabFrame();
/** \brief Gets the dsp_stream_p structure with the last frame data.
 *  @return NULL if not ready, otherwise the last dsp_stream_p filled.
 */
DLL_EXPORT dsp_stream_p iNovaSIS_GetStream();
/** \brief Returns the last exposure time requested.
 *  @return the last exposure time.
 */
DLL_EXPORT double iNovaSIS_GetExpTime();
/** \brief Requests an exposure time.
 *  \param x the exposure time in milliseconds.
 */
DLL_EXPORT void iNovaSIS_SetExpTime(double x);
/** \brief Cancels an exposure.
 */
DLL_EXPORT void iNovaSIS_CancelLongExpTime();
/** \brief Returns the last gain requested.
 *  @return the last gain.
 */
DLL_EXPORT int iNovaSIS_GetAnalogGain();
/** \brief Requests an gain value.
 *  \param AnalogGain the gain value.
 */
DLL_EXPORT void iNovaSIS_SetAnalogGain(int AnalogGain);
/** \brief Returns the maximum allowed picture height.
 *  @return the maximum allowed picture height.
 */
DLL_EXPORT int iNovaSIS_MaxImageHeight();
/** \brief Returns the maximum allowed picture width.
 *  @return the maximum allowed picture width.
 */
DLL_EXPORT int iNovaSIS_MaxImageWidth();
/** \brief Returns the current picture height.
 *  @return the current picture height.
 */
DLL_EXPORT int iNovaSIS_GetImageHeight();
/** \brief Returns the current picture width.
 *  @return the current picture width.
 */
DLL_EXPORT int iNovaSIS_GetImageWidth();
/** \brief Returns the current horizontal binning value.
 *  @return the current horizontal binning value.
 */
DLL_EXPORT int iNovaSIS_GetBinX();
/** \brief Sets the horizontal binning value.
 *  \param bin the horizontal binning value.
 */
DLL_EXPORT void iNovaSIS_SetBinX(int bin);
/** \brief Returns the current vertical binning value.
 *  @return the current vertical binning value.
 */
DLL_EXPORT int iNovaSIS_GetBinY();
/** \brief Sets the vertical binning value.
 *  \param bin the vertical binning value.
 */
DLL_EXPORT void iNovaSIS_SetBinY(int bin);
/** \brief Returns the current frame speed value.
 *  @return the current frame speed value.
 */
DLL_EXPORT inova_frame_speed_t iNovaSIS_GetFrameSpeed();
/** \brief Sets the frame speed value.
 *  \param speed the frame speed value.
 */
DLL_EXPORT void iNovaSIS_SetFrameSpeed(inova_frame_speed_t speed);
/** \brief Returns the current black level value.
 *  @return the current black level value.
 */
DLL_EXPORT int iNovaSIS_GetBlackLevel();
/** \brief Sets the black level value.
 *  \param Level the black level value.
 */
DLL_EXPORT void iNovaSIS_SetBlackLevel(int Level);
/** \brief Turns off the sensor.
 */
DLL_EXPORT void iNovaSIS_SensorPowerDown();
/** \brief Indicates if the imaging sensor is not monochrome.
 */
DLL_EXPORT int iNovaSIS_HasColorSensor();
/** \brief Returns the horizontal size of a single pixel.
 *  @return size of a single pixel.
 */
DLL_EXPORT double iNovaSIS_GetPixelSizeX();
/** \brief Returns the vertical size of a single pixel.
 *  @return size of a single pixel.
 */
DLL_EXPORT double iNovaSIS_GetPixelSizeY();
/** \brief Returns the temperature of the nth temperature chip.
 *  \param n the temperature sensor index.
 *  @return the temperature of selected chip.
 */
DLL_EXPORT double iNovaSIS_GetSensorTemperature(int n);
/*@}*/
/**
 * \defgroup SISAPI_PERIPHERAL Functions to control SIS Box peripherals
*/
/*@{*/
/** \brief Sets the pins into the ST4 autoguider port.
 *  \param n the pin mask.
 */
DLL_EXPORT void iNovaSIS_SendST4(inova_st4_direction_t n);
/** \brief Drives the GPIOs of the RJ11 port (ST4).
 *  \param n the pin mask.
 */
DLL_EXPORT void iNovaSIS_WriteRJ11(int n);
/** \brief Reads the GPIOs of the RJ9 port.
 *  \param n the pin number.
 *  @return current value of the input port.
 */
DLL_EXPORT int iNovaSIS_ReadRJ9(int n);
/** \brief Configures the selected line as GPIO.
 *  \param gpio the gpio number.
 *  \param dir the direction 1 for input 0 for output.
 */
DLL_EXPORT void iNovaSIS_OpenGpio(int gpio, int dir);
/** \brief Deconfigures the selected line as GPIO.
 *  \param gpio the gpio number.
 */
DLL_EXPORT void iNovaSIS_CloseGpio(int gpio);
/** \brief Drives the GPIOs initialized by iNovaSIS_OpenGpio().
 *  \param gpio the gpio number.
 *  \param value the value to be set on this gpio.
 *  \sa iNovaSIS_OpenGpio()
 *  \sa iNovaSIS_CloseGpio()
 */
DLL_EXPORT void iNovaSIS_SetGpio(int gpio, int value);
/** \brief Reads the GPIOs initialized by iNovaSIS_OpenGpio().
 *  \param gpio the gpio number.
 *  \sa iNovaSIS_OpenGpio()
 *  \sa iNovaSIS_CloseGpio()
 *  @return current value of the selected gpio.
 */
DLL_EXPORT int iNovaSIS_GetGpio(int gpio);
/** \brief Writes a buffer on an i2c port.
 *  \param n the index of the i2c port.
 *  \param dev_addr the address of the chip in the i2c bus.
 *  \param addr the starting address to write on the chip.
 *  \param data the data buffer to be written.
 *  \param data_len the length of the data buffer.
 *  @return 0 on success, non-zero on error.
 */
DLL_EXPORT int iNovaSIS_Writei2c(int n, int dev_addr, unsigned char addr, unsigned char* data, int data_len);
/** \brief Reads a buffer from an i2c port.
 *  \param n the index of the i2c port.
 *  \param dev_addr the address of the chip in the i2c bus.
 *  \param addr the starting address to read of the chip.
 *  \param data the data buffer to be filled.
 *  \param data_len the length of the data buffer.
 *  @return 0 on success, non-zero on error.
 */
DLL_EXPORT int iNovaSIS_Readi2c(int n, int dev_addr, unsigned char addr, unsigned char* data, int data_len);
/** \brief Opens the ir port.
 *  \param n the IR port number.
 *  @return the ir handle.
 */
DLL_EXPORT inova_ir_handle_t iNovaSIS_OpenIR(int n);
/** \brief Closes the ir port.
 *  \param ir the ir port handle.
 */
DLL_EXPORT void iNovaSIS_CloseIR(inova_ir_handle_t ir);
/** \brief Reads a byte from the ir port.
 *  \param ir the ir port handle.
 *  @return the message received.
 */
DLL_EXPORT inova_ir_data_t iNovaSIS_ReadIR(inova_ir_handle_t ir);
/*@}*/
/**
 * \defgroup SISAPI_NETWORK SIS Box network functions
*/
/*@{*/
/** \brief Starts a TCP server at Port
 *  \param Port The port on which listen for connections.
 *  \param nclients the maximum client number.
 *  \param send_callback The function to call when connecting, and that manages writing to clients.
 *  \param recv_callback The function to call when connecting, and that manages reading from clients.
 *  @return the server handle.
 */
DLL_EXPORT iNovaSIS_Server* iNovaSIS_StartServer(int Port, unsigned int nclients, inova_network_callback_t *send_callback, inova_network_callback_t *recv_callback);
/** \brief Stops the server
 *  \param server the server handle.
 */
DLL_EXPORT void iNovaSIS_StopServer(iNovaSIS_Server* server);
/** \brief Counts the number of clients connected
 *  \param server the server handle.
 *  @return the number of clients.
 */
DLL_EXPORT size_t iNovaSIS_ConnectedCount(iNovaSIS_Server* server);
/** \brief Disconnect the client specified
 *  \param client the client handle.
 */
DLL_EXPORT void iNovaSIS_CloseClient(iNovaSIS_Client* client);
/** \brief Writes a byte to the client.
 *  \param sock the client socket.
 *  \param buf the buffer to be sent.
 *  \param len the length in bytes of the message.
 *  @return the number of bytes written.
 */
DLL_EXPORT ssize_t iNovaSIS_WriteSocket(inova_socket_handle_t sock, void* buf, size_t len);
/** \brief Reads a byte from the client.
 *  \param sock the client socket.
 *  \param buf the buffer to be read.
 *  \param len the length in bytes of the message.
 *  @return the number of bytes read.
 */
DLL_EXPORT ssize_t iNovaSIS_ReadSocket(inova_socket_handle_t sock, void* buf, size_t len);
/*@}*/
/*@}*/

#ifdef  __cplusplus
}
#endif

#endif
