/**------------------------------------------------------------------------------
* rtklib.h : rtklib constants, types and function prototypes
*
*   Copyright (C) 2007-2015 by T.TAKASU, All rights reserved.
*
* options : -DENAGLO enable GLONASS
*   -DENAGAL enable Galileo
*   -DENAQZS enable QZSS
*   -DENACMP enable BeiDou
*   -DNFREQ=n  set number of obs codes/frequencies
*   -DNEXOBS=n set number of extended obs codes
*   -DMAXOBS=n set max number of obs data in an epoch
*   -DEXTLEX enable QZSS LEX extension
*
* version : $Revision: 1.1 $ $Date: 2008/07/17 21:48:06 $
* history : 2007/01/13 1.0  rtklib ver.1.0.0
*   2007/03/20 1.1  rtklib ver.1.1.0
*   2008/07/15 1.2  rtklib ver.2.1.0
*   2008/10/19 1.3  rtklib ver.2.1.1
*   2009/01/31 1.4  rtklib ver.2.2.0
*   2009/04/30 1.5  rtklib ver.2.2.1
*   2009/07/30 1.6  rtklib ver.2.2.2
*   2009/12/25 1.7  rtklib ver.2.3.0
*   2010/07/29 1.8  rtklib ver.2.4.0
*   2011/05/27 1.9  rtklib ver.2.4.1
*   2013/03/28 1.10 rtklib ver.2.4.2

***-----------------------------------------------------------------------------*/

#ifndef RTKLIB_H
#define RTKLIB_H
#ifdef __cplusplus
extern "C" {
#endif
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <ctype.h>
#ifdef _WIN32
#include <winsock2.h>
#include <windows.h>
#endif
#include <pthread.h>

#ifndef DLL_EXPORT
#define DLL_EXPORT extern
#endif

/** \defgroup RTKLIB RTKLIB: An Open Source Program Package for GNSS Positioning
*
* 
*
*  RTKLIB is an open source program package for standard and precise positioning with GNSS (global navigation satellite system). RTKLIB consists of a portable program library and several APs (application programs) utilizing the library. The features of RTKLIB are:
*
*  (1) It supports standard and precise positioning algorithms with:
*
*   GPS, GLONASS, Galileo, QZSS, BeiDou and SBAS
*
*  (2) It supports various positioning modes with GNSS for both real-time and post-processing:
*
*    Single, DGPS/DGNSS, Kinematic, Static, Moving-Baseline, Fixed, PPP-Kinematic, PPP-Staticand PPP-Fixed
*
*  (3) It supports many standard formats and protocols for GNSS:
*
*    RINEX 2.10, 2.11, 2.12 OBS/NAV/GNAV/HNAV/LNAV/QNAV, RINEX 3.00, 3.01, 3.02 OBS/NAV, RINEX 3.02 CLK, RTCM ver.2.3, RTCM ver.3.1 (with amendment 1-5), ver.3.2, BINEX, NTRIP 1.0, RTCA/DO-229C, NMEA 0183, SP3-c, ANTEX 1.4, IONEX 1.0, NGS PCV and EMS 2.0 (refer the Manual for details)
*
*  (4) It supports several GNSS receivers' proprietary messages:
*
*    NovAtel: OEM4/V/6, OEM3, OEMStar, Superstar II, Hemisphere: Eclipse, Crescent, u-blox: LEA-4T/5T/6T, SkyTraq: S1315F, JAVAD: GRIL/GREIS, Furuno: GW-10 II/III and NVS NV08C BINR (refer the Manual for details)
*
*  (6) It supports external communication via:
*
*    Serial, TCP/IP, NTRIP, local log file (record and playback) and FTP/HTTP (automatic download)
*
*  (7) It provides many library functions and APIs for GNSS data processing:
*
*    Satellite and navigation system functions, matrix and vector functions, time and string functions, coordinates transformation, input and output functions, debug trace functions, platform dependent *functions, positioning models, atmosphere models, antenna models, earth tides models, geoid models, datum transformation, RINEX functions, ephemeris and clock functions, precise ephemeris and clock functions, receiver raw data functions, RTCM functions, solution functions, Google Earth KML converter, SBAS functions, options functions, stream data input and output functions, integer ambiguity resolution, standard positioning, precise positioning, post-processing positioning, stream server functions, RTK server functions, downloader functions
*
*
* @{
*/


/**
 * \defgroup RTKLIB_Defines RTKLIB defines
*/
/*@{*/
/** library version */
#define VER_RTKLIB  "2.4.2"    

/** patch level */
#define PATCH_LEVEL "p11"    

#define COPYRIGHT_RTKLIB \
    "Copyright (C) 2007-2015 by T.Takasu\nAll rights reserved."

/** pi */
#define PI   3.1415926535897932  
/** deg to rad */
#define D2R    (PI/180.0)   
/** rad to deg */
#define R2D    (180.0/PI)   
/** speed of light (m/s) */
#define CLIGHT   299792458.0    
/** semi-circle to radian (IS-GPS) */
#define SC2RAD   3.1415926535898  
/** 1 AU (m) */
#define AU   149597870691.0   
/** arc sec to radian */
#define AS2R    (D2R/3600.0)    

/** earth angular velocity (IS-GPS) (rad/s) */
#define OMGE    7.2921151467E-5  

/** earth semimajor axis (WGS84) (m) */
#define RE_WGS84  6378137.0   
/** earth flattening (WGS84) */
#define FE_WGS84  (1.0/298.257223563) 

/** ionosphere height (m) */
#define HION    350000.0   

/** max NFREQ */
#define MAXFREQ  7    

/** L1/E1  frequency (Hz) */
#define FREQ1 1.57542E9   
/** L2  frequency (Hz) */
#define FREQ2   1.22760E9   
/** L5/E5a frequency (Hz) */
#define FREQ5   1.17645E9   
/** E6/LEX frequency (Hz) */
#define FREQ6   1.27875E9   
/** E5b  frequency (Hz) */
#define FREQ7   1.20714E9   
/** E5a+b  frequency (Hz) */
#define FREQ8   1.191795E9   
/** GLONASS G1 base frequency (Hz) */
#define FREQ1_GLO 1.60200E9   
/** GLONASS G1 bias frequency (Hz/n) */
#define DFRQ1_GLO 0.56250E6   
/** GLONASS G2 base frequency (Hz) */
#define FREQ2_GLO 1.24600E9   
/** GLONASS G2 bias frequency (Hz/n) */
#define DFRQ2_GLO 0.43750E6   
/** GLONASS G3 frequency (Hz) */
#define FREQ3_GLO 1.202025E9   
/** BeiDou B1 frequency (Hz) */
#define FREQ1_CMP 1.561098E9   
/** BeiDou B2 frequency (Hz) */
#define FREQ2_CMP 1.20714E9   
/** BeiDou B3 frequency (Hz) */
#define FREQ3_CMP 1.26852E9   

/** error factor: GPS */
#define EFACT_GPS 1.0     
/** error factor: GLONASS */
#define EFACT_GLO 1.5     
/** error factor: Galileo */
#define EFACT_GAL 1.0     
/** error factor: QZSS */
#define EFACT_QZS 1.0     
/** error factor: BeiDou */
#define EFACT_CMP 1.0     
/** error factor: SBAS */
#define EFACT_SBS 3.0     

/** navigation system: none */
#define SYS_NONE  0x00     
/** navigation system: GPS */
#define SYS_GPS  0x01     
/** navigation system: SBAS */
#define SYS_SBS  0x02     
/** navigation system: GLONASS */
#define SYS_GLO  0x04     
/** navigation system: Galileo */
#define SYS_GAL  0x08     
/** navigation system: QZSS */
#define SYS_QZS  0x10     
/** navigation system: BeiDou */
#define SYS_CMP  0x20     
/** navigation system: LEO */
#define SYS_LEO  0x40     
/** navigation system: all */
#define SYS_ALL  0xFF     

/** time system: GPS time */
#define TSYS_GPS  0     
/** time system: UTC */
#define TSYS_UTC  1     
/** time system: GLONASS time */
#define TSYS_GLO  2     
/** time system: Galileo time */
#define TSYS_GAL  3     
/** time system: QZSS time */
#define TSYS_QZS  4     
/** time system: BeiDou time */
#define TSYS_CMP  5     

#ifndef NFREQ
/** number of carrier frequencies */
#define NFREQ   3     
#endif
/** number of carrier frequencies of GLONASS */
#define NFREQGLO  2     

#ifndef NEXOBS
/** number of extended obs codes */
#define NEXOBS   0     
#endif

/** min satellite PRN number of GPS */
#define MINPRNGPS 1     
/** max satellite PRN number of GPS */
#define MAXPRNGPS 32     
/** number of GPS satellites */
#define NSATGPS  (MAXPRNGPS-MINPRNGPS+1) 
#define NSYSGPS  1

#ifdef ENAGLO
/** min satellite slot number of GLONASS */
#define MINPRNGLO 1     
/** max satellite slot number of GLONASS */
#define MAXPRNGLO 24     
/** number of GLONASS satellites */
#define NSATGLO  (MAXPRNGLO-MINPRNGLO+1) 
#define NSYSGLO  1
#else
#define MINPRNGLO 0
#define MAXPRNGLO 0
#define NSATGLO  0
#define NSYSGLO  0
#endif
#ifdef ENAGAL
/** min satellite PRN number of Galileo */
#define MINPRNGAL 1     
/** max satellite PRN number of Galileo */
#define MAXPRNGAL 27     
/** number of Galileo satellites */
#define NSATGAL  (MAXPRNGAL-MINPRNGAL+1) 
#define NSYSGAL  1
#else
#define MINPRNGAL 0
#define MAXPRNGAL 0
#define NSATGAL  0
#define NSYSGAL  0
#endif
#ifdef ENAQZS
/** min satellite PRN number of QZSS */
#define MINPRNQZS 193     
/** max satellite PRN number of QZSS */
#define MAXPRNQZS 199     
/** min satellite PRN number of QZSS SAIF */
#define MINPRNQZS_S 183     
/** max satellite PRN number of QZSS SAIF */
#define MAXPRNQZS_S 189     
/** number of QZSS satellites */
#define NSATQZS  (MAXPRNQZS-MINPRNQZS+1) 
#define NSYSQZS  1
#else
#define MINPRNQZS 0
#define MAXPRNQZS 0
#define MINPRNQZS_S 0
#define MAXPRNQZS_S 0
#define NSATQZS  0
#define NSYSQZS  0
#endif
#ifdef ENACMP
/** min satellite sat number of BeiDou */
#define MINPRNCMP 1     
/** max satellite sat number of BeiDou */
#define MAXPRNCMP 35     
/** number of BeiDou satellites */
#define NSATCMP  (MAXPRNCMP-MINPRNCMP+1) 
#define NSYSCMP  1
#else
#define MINPRNCMP 0
#define MAXPRNCMP 0
#define NSATCMP  0
#define NSYSCMP  0
#endif
#ifdef ENALEO
/** min satellite sat number of LEO */
#define MINPRNLEO 1     
/** max satellite sat number of LEO */
#define MAXPRNLEO 10     
/** number of LEO satellites */
#define NSATLEO  (MAXPRNLEO-MINPRNLEO+1) 
#define NSYSLEO  1
#else
#define MINPRNLEO 0
#define MAXPRNLEO 0
#define NSATLEO  0
#define NSYSLEO  0
#endif
/** number of systems */
#define NSYS    (NSYSGPS+NSYSGLO+NSYSGAL+NSYSQZS+NSYSCMP+NSYSLEO) 

/** min satellite PRN number of SBAS */
#define MINPRNSBS 120     
/** max satellite PRN number of SBAS */
#define MAXPRNSBS 142     
/** number of SBAS satellites */
#define NSATSBS  (MAXPRNSBS-MINPRNSBS+1) 

/** max satellite number (1 to MAXSAT) */
#define MAXSAT   (NSATGPS+NSATGLO+NSATGAL+NSATQZS+NSATCMP+NSATSBS+NSATLEO)
           
#ifndef MAXOBS
/** max number of obs in an epoch */
#define MAXOBS   64     
#endif
/** max receiver number (1 to MAXRCV) */
#define MAXRCV   64     
/** max number of obs type in RINEX */
#define MAXOBSTYPE  64     
/** tolerance of time difference (s) */
#define DTTOL   0.005    
/** max time difference to GPS Toe (s) */
#define MAXDTOE  7200.0    
/** max time difference to QZSS Toe (s) */
#define MAXDTOE_QZS 7200.0    
/** max time difference to Galileo Toe (s) */
#define MAXDTOE_GAL 10800.0    
/** max time difference to BeiDou Toe (s) */
#define MAXDTOE_CMP 21600.0    
/** max time difference to GLONASS Toe (s) */
#define MAXDTOE_GLO 1800.0    
/** max time difference to SBAS Toe (s) */
#define MAXDTOE_SBS 360.0    
/** max time difference to ephem toe (s) for other */
#define MAXDTOE_S 86400.0    
/** max GDOP */
#define MAXGDOP  300.0    

/** swap interval of trace file (s) */
#define INT_SWAP_TRAC 86400.0   
/** swap interval of solution status file (s) */
#define INT_SWAP_STAT 86400.0   

/** max number of expanded files */
#define MAXEXFILE 1024     
/** max age of SBAS fast correction (s) */
#define MAXSBSAGEF  30.0     
/** max age of SBAS long term corr (s) */
#define MAXSBSAGEL  1800.0    
/** max URA of SBAS satellite */
#define MAXSBSURA 8     
/** max SBAS band of IGP */
#define MAXBAND  10     
/** max number of IGP in SBAS band */
#define MAXNIGP  201     
/** max number of GEO satellites */
#define MAXNGEO  4     
/** max number of RINEX comments */
#define MAXCOMMENT  10     
/** max length of stream path */
#define MAXSTRPATH  1024     
/** max length of stream message */
#define MAXSTRMSG 1024     
/** max number of stream in RTK server */
#define MAXSTRRTK 8     
/** max number of SBAS msg in RTK server */
#define MAXSBSMSG 32     
/** max length of solution message */
#define MAXSOLMSG 4096     
/** max length of receiver raw message */
#define MAXRAWLEN 4096     
/** max length of error/warning message */
#define MAXERRMSG 4096     
/** max length of station name/antenna type */
#define MAXANT   64     
/** max number of solution buffer */
#define MAXSOLBUF 256     
/** max number of observation data buffer */
#define MAXOBSBUF 128     
/** max number of reference positions */
#define MAXNRPOS  16     
/** max number of leap seconds table */
#define MAXLEAPS  64     

/** RINEX ver.2 default output version */
#define RNX2VER  2.10     
/** RINEX ver.3 default output version */
#define RNX3VER  3.00     

/** observation type: pseudorange */
#define OBSTYPE_PR  0x01     
/** observation type: carrier-phase */
#define OBSTYPE_CP  0x02     
/** observation type: doppler-freq */
#define OBSTYPE_DOP 0x04     
/** observation type: SNR */
#define OBSTYPE_SNR 0x08     
/** observation type: all */
#define OBSTYPE_ALL 0xFF     

/** frequency type: L1/E1 */
#define FREQTYPE_L1 0x01     
/** frequency type: L2/B1 */
#define FREQTYPE_L2 0x02     
/** frequency type: L5/E5a/L3 */
#define FREQTYPE_L5 0x04     
/** frequency type: E6/LEX/B3 */
#define FREQTYPE_L6 0x08     
/** frequency type: E5b/B2 */
#define FREQTYPE_L7 0x10     
/** frequency type: E5(a+b) */
#define FREQTYPE_L8 0x20     
/** frequency type: all */
#define FREQTYPE_ALL 0xFF    

/** obs code: none or unknown */
#define CODE_NONE 0     
/** obs code: L1C/A,G1C/A,E1C (GPS,GLO,GAL,QZS,SBS) */
#define CODE_L1C  1     
/** obs code: L1P,G1P  (GPS,GLO) */
#define CODE_L1P  2     
/** obs code: L1 Z-track (GPS) */
#define CODE_L1W  3     
/** obs code: L1Y    (GPS) */
#define CODE_L1Y  4     
/** obs code: L1M    (GPS) */
#define CODE_L1M  5     
/** obs code: L1codeless (GPS) */
#define CODE_L1N  6     
/** obs code: L1C(D)  (GPS,QZS) */
#define CODE_L1S  7     
/** obs code: L1C(P)  (GPS,QZS) */
#define CODE_L1L  8     
/** obs code: L1-SAIF  (QZS) */
#define CODE_L1E  9     
/** obs code: E1A    (GAL) */
#define CODE_L1A  10     
/** obs code: E1B    (GAL) */
#define CODE_L1B  11     
/** obs code: E1B+C,L1C(D+P) (GAL,QZS) */
#define CODE_L1X  12     
/** obs code: E1A+B+C,L1SAIF (GAL,QZS) */
#define CODE_L1Z  13     
/** obs code: L2C/A,G1C/A (GPS,GLO) */
#define CODE_L2C  14     
/** obs code: L2 L1C/A-(P2-P1) (GPS) */
#define CODE_L2D  15     
/** obs code: L2C(M)  (GPS,QZS) */
#define CODE_L2S  16     
/** obs code: L2C(L)  (GPS,QZS) */
#define CODE_L2L  17     
/** obs code: L2C(M+L),B1I+Q (GPS,QZS,CMP) */
#define CODE_L2X  18     
/** obs code: L2P,G2P  (GPS,GLO) */
#define CODE_L2P  19     
/** obs code: L2 Z-track (GPS) */
#define CODE_L2W  20     
/** obs code: L2Y    (GPS) */
#define CODE_L2Y  21     
/** obs code: L2M    (GPS) */
#define CODE_L2M  22     
/** obs code: L2codeless (GPS) */
#define CODE_L2N  23     
/** obs code: L5/E5aI  (GPS,GAL,QZS,SBS) */
#define CODE_L5I  24     
/** obs code: L5/E5aQ  (GPS,GAL,QZS,SBS) */
#define CODE_L5Q  25     
/** obs code: L5/E5aI+Q  (GPS,GAL,QZS,SBS) */
#define CODE_L5X  26     
/** obs code: E5bI,B2I (GAL,CMP) */
#define CODE_L7I  27     
/** obs code: E5bQ,B2Q (GAL,CMP) */
#define CODE_L7Q  28     
/** obs code: E5bI+Q,B2I+Q (GAL,CMP) */
#define CODE_L7X  29     
/** obs code: E6A    (GAL) */
#define CODE_L6A  30     
/** obs code: E6B    (GAL) */
#define CODE_L6B  31     
/** obs code: E6C    (GAL) */
#define CODE_L6C  32     
/** obs code: E6B+C,LEXS+L,B3I+Q (GAL,QZS,CMP) */
#define CODE_L6X  33     
/** obs code: E6A+B+C  (GAL) */
#define CODE_L6Z  34     
/** obs code: LEXS   (QZS) */
#define CODE_L6S  35     
/** obs code: LEXL   (QZS) */
#define CODE_L6L  36     
/** obs code: E5(a+b)I (GAL) */
#define CODE_L8I  37     
/** obs code: E5(a+b)Q (GAL) */
#define CODE_L8Q  38     
/** obs code: E5(a+b)I+Q (GAL) */
#define CODE_L8X  39     
/** obs code: B1I    (CMP) */
#define CODE_L2I  40     
/** obs code: B1Q    (CMP) */
#define CODE_L2Q  41     
/** obs code: B3I    (CMP) */
#define CODE_L6I  42     
/** obs code: B3Q    (CMP) */
#define CODE_L6Q  43     
/** obs code: G3I    (GLO) */
#define CODE_L3I  44     
/** obs code: G3Q    (GLO) */
#define CODE_L3Q  45     
/** obs code: G3I+Q   (GLO) */
#define CODE_L3X  46     
/** obs code: B1I    (BDS) */
#define CODE_L1I  47     
/** obs code: B1Q    (BDS) */
#define CODE_L1Q  48     
/** max number of obs code */
#define MAXCODE  48     

/** positioning mode: single */
#define PMODE_SINGLE 0     
/** positioning mode: DGPS/DGNSS */
#define PMODE_DGPS 1     
/** positioning mode: kinematic */
#define PMODE_KINEMA 2     
/** positioning mode: static */
#define PMODE_STATIC 3     
/** positioning mode: moving-base */
#define PMODE_MOVEB  4     
/** positioning mode: fixed */
#define PMODE_FIXED  5     
/** positioning mode: PPP-kinemaric */
#define PMODE_PPP_KINEMA 6    
/** positioning mode: PPP-static */
#define PMODE_PPP_STATIC 7    
/** positioning mode: PPP-fixed */
#define PMODE_PPP_FIXED 8    

/** solution format: lat/lon/height */
#define SOLF_LLH  0     
/** solution format: x/y/z-ecef */
#define SOLF_XYZ  1     
/** solution format: e/n/u-baseline */
#define SOLF_ENU  2     
/** solution format: NMEA-183 */
#define SOLF_NMEA 3     
/** solution format: GSI-F1/2/3 */
#define SOLF_GSIF 4     

/** solution status: no solution */
#define SOLQ_NONE 0     
/** solution status: fix */
#define SOLQ_FIX  1     
/** solution status: float */
#define SOLQ_FLOAT  2     
/** solution status: SBAS */
#define SOLQ_SBAS 3     
/** solution status: DGPS/DGNSS */
#define SOLQ_DGPS 4     
/** solution status: single */
#define SOLQ_SINGLE 5     
/** solution status: PPP */
#define SOLQ_PPP  6     
/** solution status: dead reconing */
#define SOLQ_DR  7     
/** max number of solution status */
#define MAXSOLQ  7     

/** time system: gps time */
#define TIMES_GPST  0     
/** time system: utc */
#define TIMES_UTC 1     
/** time system: jst */
#define TIMES_JST 2     

/** ionosphere option: correction off */
#define IONOOPT_OFF 0     
/** ionosphere option: broadcast model */
#define IONOOPT_BRDC 1     
/** ionosphere option: SBAS model */
#define IONOOPT_SBAS 2     
/** ionosphere option: L1/L2 or L1/L5 iono-free LC */
#define IONOOPT_IFLC 3     
/** ionosphere option: estimation */
#define IONOOPT_EST 4     
/** ionosphere option: IONEX TEC model */
#define IONOOPT_TEC 5     
/** ionosphere option: QZSS broadcast model */
#define IONOOPT_QZS 6     
/** ionosphere option: QZSS LEX ionospehre */
#define IONOOPT_LEX 7     
/** ionosphere option: SLANT TEC model */
#define IONOOPT_STEC 8     

/** troposphere option: correction off */
#define TROPOPT_OFF 0     
/** troposphere option: Saastamoinen model */
#define TROPOPT_SAAS 1     
/** troposphere option: SBAS model */
#define TROPOPT_SBAS 2     
/** troposphere option: ZTD estimation */
#define TROPOPT_EST 3     
/** troposphere option: ZTD+grad estimation */
#define TROPOPT_ESTG 4     
/** troposphere option: ZTD correction */
#define TROPOPT_COR 5     
/** troposphere option: ZTD+grad correction */
#define TROPOPT_CORG 6     

/** ephemeris option: broadcast ephemeris */
#define EPHOPT_BRDC 0     
/** ephemeris option: precise ephemeris */
#define EPHOPT_PREC 1     
/** ephemeris option: broadcast + SBAS */
#define EPHOPT_SBAS 2     
/** ephemeris option: broadcast + SSR_APC */
#define EPHOPT_SSRAPC 3     
/** ephemeris option: broadcast + SSR_COM */
#define EPHOPT_SSRCOM 4     
/** ephemeris option: QZSS LEX ephemeris */
#define EPHOPT_LEX  5     

/** AR mode: off */
#define ARMODE_OFF  0     
/** AR mode: continuous */
#define ARMODE_CONT 1     
/** AR mode: instantaneous */
#define ARMODE_INST 2     
/** AR mode: fix and hold */
#define ARMODE_FIXHOLD 3     
/** AR mode: PPP-AR */
#define ARMODE_PPPAR 4     
/** AR mode: PPP-AR ILS */
#define ARMODE_PPPAR_ILS 5    
/** AR mode: wide lane/narrow lane */
#define ARMODE_WLNL 6     
/** AR mode: triple carrier ar */
#define ARMODE_TCAR 7     

/** SBAS option: long term correction */
#define SBSOPT_LCORR 1     
/** SBAS option: fast correction */
#define SBSOPT_FCORR 2     
/** SBAS option: ionosphere correction */
#define SBSOPT_ICORR 4     
/** SBAS option: ranging */
#define SBSOPT_RANGE 8     

/** stream type: none */
#define STR_NONE  0     
/** stream type: serial */
#define STR_SERIAL 1     
/** stream type: file */
#define STR_FILE  2     
/** stream type: TCP server */
#define STR_TCPSVR 3     
/** stream type: TCP client */
#define STR_TCPCLI 4     
/** stream type: UDP stream */
#define STR_UDP   5     
/** stream type: NTRIP server */
#define STR_NTRIPSVR 6     
/** stream type: NTRIP client */
#define STR_NTRIPCLI 7     
/** stream type: ftp */
#define STR_FTP   8     
/** stream type: http */
#define STR_HTTP  9     
/** stream type: unix domain socket server */
#define STR_UNIXSVR  10     

/** stream format: RTCM 2 */
#define STRFMT_RTCM2 0     
/** stream format: RTCM 3 */
#define STRFMT_RTCM3 1     
/** stream format: NovAtel OEMV/4 */
#define STRFMT_OEM4  2     
/** stream format: NovAtel OEM3 */
#define STRFMT_OEM3  3     
/** stream format: u-blox LEA-*T */
#define STRFMT_UBX 4     
/** stream format: NovAtel Superstar II */
#define STRFMT_SS2 5     
/** stream format: Hemisphere */
#define STRFMT_CRES  6     
/** stream format: SkyTraq S1315F */
#define STRFMT_STQ 7     
/** stream format: Furuno GW10 */
#define STRFMT_GW10  8     
/** stream format: JAVAD GRIL/GREIS */
#define STRFMT_JAVAD 9     
/** stream format: NVS NVC08C */
#define STRFMT_NVS 10     
/** stream format: BINEX */
#define STRFMT_BINEX 11     
/** stream format: Trimble RT17 */
#define STRFMT_RT17  12     
/** stream format: SiRF */
#define STRFMT_SIRF  13     
/** stream format: Furuno LPY-10000 */
#define STRFMT_LEXR  14     
/** stream format: Septentrio */
#define STRFMT_SEPT  15     
/** stream format: RINEX */
#define STRFMT_RINEX 16     
/** stream format: SP3 */
#define STRFMT_SP3 17     
/** stream format: RINEX CLK */
#define STRFMT_RNXCLK 18     
/** stream format: SBAS messages */
#define STRFMT_SBAS  19     
/** stream format: NMEA 0183 */
#define STRFMT_NMEA  20     
#ifndef EXTLEX
/** max number of receiver format */
#define MAXRCVFMT  13     
#else
#define MAXRCVFMT  14
#endif

/** stream mode: read */
#define STR_MODE_R  0x1     
/** stream mode: write */
#define STR_MODE_W  0x2     
/** stream mode: read/write */
#define STR_MODE_RW 0x3     

/** geoid model: embedded geoid */
#define GEOID_EMBEDDED  0    
/** geoid model: EGM96 15x15" */
#define GEOID_EGM96_M150  1    
/** geoid model: EGM2008 2.5x2.5" */
#define GEOID_EGM2008_M25 2    
/** geoid model: EGM2008 1.0x1.0" */
#define GEOID_EGM2008_M10 3    
/** geoid model: GSI geoid 2000 1.0x1.5" */
#define GEOID_GSI2000_M15 4    
/** geoid model: IGN RAF09 for France 1.5"x2" */
#define GEOID_RAF09   5    

/** comment line indicator for solution */
#define COMMENTH  "%"     
/** disconnect message */
#define MSG_DISCONN "$_DISCONNECT\r\n"  

/** download option: force download existing */
#define DLOPT_FORCE 0x01    
/** download option: keep compressed file */
#define DLOPT_KEEPCMP 0x02    
/** download option: hold on error file */
#define DLOPT_HOLDERR 0x04    
/** download option: hold on listing file */
#define DLOPT_HOLDLST 0x08    

/** 2^-5 */
#define P2_5    0.03125    
/** 2^-6 */
#define P2_6    0.015625    
/** 2^-11 */
#define P2_11   4.882812500000000E-04 
/** 2^-15 */
#define P2_15   3.051757812500000E-05 
/** 2^-17 */
#define P2_17   7.629394531250000E-06 
/** 2^-19 */
#define P2_19   1.907348632812500E-06 
/** 2^-20 */
#define P2_20   9.536743164062500E-07 
/** 2^-21 */
#define P2_21   4.768371582031250E-07 
/** 2^-23 */
#define P2_23   1.192092895507810E-07 
/** 2^-24 */
#define P2_24   5.960464477539063E-08 
/** 2^-27 */
#define P2_27   7.450580596923828E-09 
/** 2^-29 */
#define P2_29   1.862645149230957E-09 
/** 2^-30 */
#define P2_30   9.313225746154785E-10 
/** 2^-31 */
#define P2_31   4.656612873077393E-10 
/** 2^-32 */
#define P2_32   2.328306436538696E-10 
/** 2^-33 */
#define P2_33   1.164153218269348E-10 
/** 2^-35 */
#define P2_35   2.910383045673370E-11 
/** 2^-38 */
#define P2_38   3.637978807091710E-12 
/** 2^-39 */
#define P2_39   1.818989403545856E-12 
/** 2^-40 */
#define P2_40   9.094947017729280E-13 
/** 2^-43 */
#define P2_43   1.136868377216160E-13 
/** 2^-48 */
#define P2_48   3.552713678800501E-15 
/** 2^-50 */
#define P2_50   8.881784197001252E-16 
/** 2^-55 */
#define P2_55   2.775557561562891E-17 

#ifdef _WIN32
#define FILEPATHSEP '\\'
#else
#define FILEPATHSEP '/'
#endif

#define thread_t  pthread_t
#define lock_t   pthread_mutex_t
#define initlock(f) pthread_mutex_init(f,NULL)
#define lock(f)  pthread_mutex_lock(f)
#define unlock(f) pthread_mutex_unlock(f)


/** @}
* @defgroup  RTKLIB_type type definitions 
* @{
*/

/** time struct */
typedef struct {    
/** time (s) expressed by standard time_t */
  time_t time;    
/** fraction of second under 1 s */
  double sec;    
} gtime_t;

/** observation data record */
typedef struct {    
/** receiver sampling time (GPST) */
  gtime_t time;   
/** satellite/receiver number */
  unsigned char sat,rcv; 
/** signal strength (0.25 dBHz) */
  unsigned char SNR [NFREQ+NEXOBS]; 
/** loss of lock indicator */
  unsigned char LLI [NFREQ+NEXOBS]; 
/** code indicator (CODE_???) */
  unsigned char code[NFREQ+NEXOBS]; 
/** observation data carrier-phase (cycle) */
  double L[NFREQ+NEXOBS]; 
/** observation data pseudorange (m) */
  double P[NFREQ+NEXOBS]; 
/** observation data doppler frequency (Hz) */
  float  D[NFREQ+NEXOBS]; 
} obsd_t;

/** observation data */
typedef struct {    
/** number of obervation data/allocated */
  int n,nmax;    
/** observation data records */
  obsd_t *data;   
} obs_t;

/** earth rotation parameter data type */
typedef struct {    
/** mjd (days) */
  double mjd;    
/** pole offset (rad) */
  double xp,yp;   
/** pole offset rate (rad/day) */
  double xpr,ypr;  
/** ut1-utc (s) */
  double ut1_utc;  
/** length of day (s/day) */
  double lod;    
} erpd_t;

/** earth rotation parameter type */
typedef struct {    
/** number and max number of data */
  int n,nmax;    
/** earth rotation parameter data */
  erpd_t *data;   
} erp_t;

/** antenna parameter type */
typedef struct {    
/** satellite number (0:receiver) */
  int sat;    
/** antenna type */
  char type[MAXANT];  
/** serial number or satellite code */
  char code[MAXANT];  
/** valid time start and end */
  gtime_t ts,te;   
/** phase center offset e/n/u or x/y/z (m) */
  double off[NFREQ][ 3]; 
/** phase center variation (m) */
  double var[NFREQ][19]; 
/** el=90,85,...,0 or nadir=0,1,2,3,... (deg) */
       
} pcv_t;

/** antenna parameters type */
typedef struct {    
/** number of data/allocated */
  int n,nmax;    
/** antenna parameters data */
  pcv_t *pcv;    
} pcvs_t;

/** almanac type */
typedef struct {    
/** satellite number */
  int sat;    
/** sv health (0:ok) */
  int svh;    
/** as and sv config */
  int svconf;    
/** GPS/QZS: gps week, GAL: galileo week */
  int week;   
/** Toa */
  gtime_t toa;    
/** SV orbit parameters */
       
  double A,e,i0,OMG0,omg,M0,OMGd;
/** Toa (s) in week */
  double toas;    
/** SV clock parameters (af0,af1) */
  double f0,f1;   
} alm_t;

/** GPS/QZS/GAL broadcast ephemeris type */
typedef struct {    
/** satellite number */
  int sat;    
/** IODE,IODC */
  int iode,iodc;   
/** SV accuracy (URA index) */
  int sva;    
/** SV health (0:ok) */
  int svh;    
/** GPS/QZS: gps week, GAL: galileo week */
  int week;   
/** GPS/QZS: code on L2, GAL/CMP: data sources */
  int code;   
/** GPS/QZS: L2 P data flag, CMP: nav type */
  int flag;   
/** Toe,Toc,T_trans */
  gtime_t toe,toc,ttr; 
/** SV orbit parameters */
       
  double A,e,i0,OMG0,omg,M0,deln,OMGd,idot;
  double crc,crs,cuc,cus,cic,cis;
/** Toe (s) in week */
  double toes;    
/** fit interval (h) */
  double fit;    
/** SV clock parameters (af0,af1,af2) */
  double f0,f1,f2;  
/** group delay parameters */
  double tgd[4];   
/** GPS/QZS:tgd[0]=TGD */
       
/** GAL  :tgd[0]=BGD E5a/E1,tgd[1]=BGD E5b/E1 */
       
/** CMP  :tgd[0]=BGD1,tgd[1]=BGD2 */
       
/** Adot,ndot for CNAV */
  double Adot,ndot; 
} eph_t;

/** GLONASS broadcast ephemeris type */
typedef struct {    
/** satellite number */
  int sat;    
/** IODE (0-6 bit of tb field) */
  int iode;   
/** satellite frequency number */
  int frq;    
/** satellite health, accuracy, age of operation */
  int svh,sva,age;  
/** epoch of epherides (gpst) */
  gtime_t toe;    
/** message frame time (gpst) */
  gtime_t tof;    
/** satellite position (ecef) (m) */
  double pos[3];   
/** satellite velocity (ecef) (m/s) */
  double vel[3];   
/** satellite acceleration (ecef) (m/s^2) */
  double acc[3];   
/** SV clock bias (s)/relative freq bias */
  double taun,gamn; 
/** delay between L1 and L2 (s) */
  double dtaun;   
} geph_t;

/** precise ephemeris type */
typedef struct {    
/** time (GPST) */
  gtime_t time;   
/** ephemeris index for multiple files */
  int index;   
/** satellite position/clock (ecef) (m|s) */
  double pos[MAXSAT][4]; 
/** satellite position/clock std (m|s) */
  float  std[MAXSAT][4]; 
/** satellite velocity/clk-rate (m/s|s/s) */
  double vel[MAXSAT][4]; 
/** satellite velocity/clk-rate std (m/s|s/s) */
  float  vst[MAXSAT][4]; 
/** satellite position covariance (m^2) */
  float  cov[MAXSAT][3]; 
/** satellite velocity covariance (m^2) */
  float  vco[MAXSAT][3]; 
} peph_t;

/** precise clock type */
typedef struct {    
/** time (GPST) */
  gtime_t time;   
/** clock index for multiple files */
  int index;   
/** satellite clock (s) */
  double clk[MAXSAT][1]; 
/** satellite clock std (s) */
  float  std[MAXSAT][1]; 
} pclk_t;

/** SBAS ephemeris type */
typedef struct {    
/** satellite number */
  int sat;    
/** reference epoch time (GPST) */
  gtime_t t0;    
/** time of message frame (GPST) */
  gtime_t tof;    
/** SV accuracy (URA index) */
  int sva;    
/** SV health (0:ok) */
  int svh;    
/** satellite position (m) (ecef) */
  double pos[3];   
/** satellite velocity (m/s) (ecef) */
  double vel[3];   
/** satellite acceleration (m/s^2) (ecef) */
  double acc[3];   
/** satellite clock-offset/drift (s,s/s) */
  double af0,af1;  
} seph_t;

/** norad two line element data type */
typedef struct {    
/** common name */
  char name [32];  
/** alias name */
  char alias[32];  
/** satellilte catalog number */
  char satno[16];  
/** classification */
  char satclass;   
/** international designator */
  char desig[16];  
/** element set epoch (UTC) */
  gtime_t epoch;   
/** 1st derivative of mean motion */
  double ndot;    
/** 2st derivative of mean motion */
  double nddot;   
/** B* drag term */
  double bstar;   
/** element set type */
  int etype;   
/** element number */
  int eleno;   
/** orbit inclination (deg) */
  double inc;    
/** right ascension of ascending node (deg) */
  double OMG;    
/** eccentricity */
  double ecc;    
/** argument of perigee (deg) */
  double omg;    
/** mean anomaly (deg) */
  double M;   
/** mean motion (rev/day) */
  double n;   
/** revolution number at epoch */
  int rev;    
} tled_t;

/** norad two line element type */
typedef struct {    
/** number/max number of two line element data */
  int n,nmax;    
/** norad two line element data */
  tled_t *data;   
} tle_t;

/** TEC grid type */
typedef struct {    
/** epoch time (GPST) */
  gtime_t time;   
/** TEC grid data size {nlat,nlon,nhgt} */
  int ndata[3];   
/** earth radius (km) */
  double rb;   
/** latitude start/interval (deg) */
  double lats[3];  
/** longitude start/interval (deg) */
  double lons[3];  
/** heights start/interval (km) */
  double hgts[3];  
/** TEC grid data (tecu) */
  double *data;   
/** RMS values (tecu) */
  float *rms;    
} tec_t;

/** stec data type */
typedef struct {    
/** time (GPST) */
  gtime_t time;   
/** satellite number */
  unsigned char sat;  
/** slip flag */
  unsigned char slip; 
/** L1 ionosphere delay (m) */
  float iono;    
/** L1 ionosphere rate (m/s) */
  float rate;    
/** rms value (m) */
  float rms;   
} stecd_t;

/** stec grid type */
typedef struct {    
/** latitude/longitude (deg) */
  double pos[2];   
/** search index */
  int index[MAXSAT];  
/** number of data */
  int n,nmax;    
/** stec data */
  stecd_t *data;   
} stec_t;

/** zwd data type */
typedef struct {    
/** time (GPST) */
  gtime_t time;   
/** zenith wet delay (m) */
  float zwd;   
/** rms value (m) */
  float rms;   
} zwdd_t;

/** zwd grid type */
typedef struct {    
/** latitude,longitude (rad) */
  float pos[2];   
/** number of data */
  int n,nmax;    
/** zwd data */
  zwdd_t *data;   
} zwd_t;

/** SBAS message type */
typedef struct {    
/** receiption time */
  int week,tow;   
/** SBAS satellite PRN number */
  int prn;    
/** SBAS message (226bit) padded by 0 */
  unsigned char msg[29]; 
} sbsmsg_t;

/** SBAS messages type */
typedef struct {    
/** number of SBAS messages/allocated */
  int n,nmax;    
/** SBAS messages */
  sbsmsg_t *msgs;  
} sbs_t;

/** SBAS fast correction type */
typedef struct {    
/** time of applicability (TOF) */
  gtime_t t0;    
/** pseudorange correction (PRC) (m) */
  double prc;    
/** range-rate correction (RRC) (m/s) */
  double rrc;    
/** range-rate correction delta-time (s) */
  double dt;   
/** IODF (issue of date fast corr) */
  int iodf;   
/** UDRE+1 */
  short udre;    
/** degradation factor indicator */
  short ai;   
} sbsfcorr_t;

/** SBAS long term satellite error correction type */
typedef struct {    
/** correction time */
  gtime_t t0;    
/** IODE (issue of date ephemeris) */
  int iode;   
/** delta position (m) (ecef) */
  double dpos[3];  
/** delta velocity (m/s) (ecef) */
  double dvel[3];  
/** delta clock-offset/drift (s,s/s) */
  double daf0,daf1; 
} sbslcorr_t;

/** SBAS satellite correction type */
typedef struct {    
/** satellite number */
  int sat;    
/** fast correction */
  sbsfcorr_t fcorr; 
/** long term correction */
  sbslcorr_t lcorr; 
} sbssatp_t;

/** SBAS satellite corrections type */
typedef struct {    
/** IODP (issue of date mask) */
  int iodp;   
/** number of satellites */
  int nsat;   
/** system latency (s) */
  int tlat;   
/** satellite correction */
  sbssatp_t sat[MAXSAT]; 
} sbssat_t;

/** SBAS ionospheric correction type */
typedef struct {    
/** correction time */
  gtime_t t0;    
/** latitude/longitude (deg) */
  short lat,lon;   
/** GIVI+1 */
  short give;    
/** vertical delay estimate (m) */
  float delay;    
} sbsigp_t;

/** IGP band type */
typedef struct {    
/** longitude/latitude (deg) */
  short x;    
/** latitudes/longitudes (deg) */
  const short *y;  
/** IGP mask start bit */
  unsigned char bits; 
/** IGP mask end bit */
  unsigned char bite; 
} sbsigpband_t;

/** SBAS ionospheric corrections type */
typedef struct {    
/** IODI (issue of date ionos corr) */
  int iodi;   
/** number of igps */
  int nigp;   
/** ionospheric correction */
  sbsigp_t igp[MAXNIGP]; 
} sbsion_t;

/** DGPS/GNSS correction type */
typedef struct {    
/** correction time */
  gtime_t t0;    
/** pseudorange correction (PRC) (m) */
  double prc;    
/** range rate correction (RRC) (m/s) */
  double rrc;    
/** issue of data (IOD) */
  int iod;    
/** UDRE */
  double udre;    
} dgps_t;

/** SSR correction type */
typedef struct {    
/** epoch time (GPST) {eph,clk,hrclk,ura,bias} */
  gtime_t t0[5];   
/** SSR update interval (s) */
  double udi[5];   
/** iod ssr {eph,clk,hrclk,ura,bias} */
  int iod[5];    
/** issue of data */
  int iode;   
/** issue of data crc for beidou/sbas */
  int iodcrc;    
/** URA indicator */
  int ura;    
/** sat ref datum (0:ITRF,1:regional) */
  int refd;   
/** delta orbit {radial,along,cross} (m) */
  double deph [3];  
/** dot delta orbit {radial,along,cross} (m/s) */
  double ddeph[3];  
/** delta clock {c0,c1,c2} (m,m/s,m/s^2) */
  double dclk [3];  
/** high-rate clock corection (m) */
  double hrclk;   
/** code biases (m) */
  float cbias[MAXCODE]; 
/** update flag (0:no update,1:update) */
  unsigned char update; 
} ssr_t;

/** QZSS LEX message type */
typedef struct {    
/** satellite PRN number */
  int prn;    
/** message type */
  int type;   
/** alert flag */
  int alert;   
/** signal tracking status */
  unsigned char stat; 
/** signal C/N0 (0.25 dBHz) */
  unsigned char snr;  
/** tracking time (ms) */
  unsigned int ttt; 
/** LEX message data part 1695 bits */
  unsigned char msg[212]; 
} lexmsg_t;

/** QZSS LEX messages type */
typedef struct {    
/** number of LEX messages and allocated */
  int n,nmax;    
/** LEX messages */
  lexmsg_t *msgs;  
} lex_t;

/** QZSS LEX ephemeris type */
typedef struct {    
/** epoch time (GPST) */
  gtime_t toe;    
/** message frame time (GPST) */
  gtime_t tof;    
/** satellite number */
  int sat;    
/** signal health (L1,L2,L1C,L5,LEX) */
  unsigned char health; 
/** URA index */
  unsigned char ura;  
/** satellite position (m) */
  double pos[3];   
/** satellite velocity (m/s) */
  double vel[3];   
/** satellite acceleration (m/s2) */
  double acc[3];   
/** satellite jerk (m/s3) */
  double jerk[3];  
/** satellite clock bias and drift (s,s/s) */
  double af0,af1;  
/** TGD */
  double tgd;    
/** ISC */
  double isc[8];   
} lexeph_t;

/** QZSS LEX ionosphere correction type */
typedef struct {    
/** epoch time (GPST) */
  gtime_t t0;    
/** valid time span (s) */
  double tspan;   
/** reference position {lat,lon} (rad) */
  double pos0[2];  
/** coefficients lat x lon (3 x 2) */
  double coef[3][2];  
} lexion_t;

/** navigation data type */
typedef struct {    
/** number of broadcast ephemeris */
  int n,nmax;    
/** number of glonass ephemeris */
  int ng,ngmax;   
/** number of sbas ephemeris */
  int ns,nsmax;   
/** number of precise ephemeris */
  int ne,nemax;   
/** number of precise clock */
  int nc,ncmax;   
/** number of almanac data */
  int na,namax;   
/** number of tec grid data */
  int nt,ntmax;   
/** number of stec grid data */
  int nn,nnmax;   
/** GPS/QZS/GAL ephemeris */
  eph_t *eph;    
/** GLONASS ephemeris */
  geph_t *geph;   
/** SBAS ephemeris */
  seph_t *seph;   
/** precise ephemeris */
  peph_t *peph;   
/** precise clock */
  pclk_t *pclk;   
/** almanac data */
  alm_t *alm;    
/** tec grid data */
  tec_t *tec;    
/** stec grid data */
  stec_t *stec;   
/** earth rotation parameters */
  erp_t  erp;    
/** GPS delta-UTC parameters {A0,A1,T,W} */
  double utc_gps[4];  
/** GLONASS UTC GPS time parameters */
  double utc_glo[4];  
/** Galileo UTC GPS time parameters */
  double utc_gal[4];  
/** QZS UTC GPS time parameters */
  double utc_qzs[4];  
/** BeiDou UTC parameters */
  double utc_cmp[4];  
/** SBAS UTC parameters */
  double utc_sbs[4];  
/** GPS iono model parameters {a0,a1,a2,a3,b0,b1,b2,b3} */
  double ion_gps[8];  
/** Galileo iono model parameters {ai0,ai1,ai2,0} */
  double ion_gal[4];  
/** QZSS iono model parameters {a0,a1,a2,a3,b0,b1,b2,b3} */
  double ion_qzs[8];  
/** BeiDou iono model parameters {a0,a1,a2,a3,b0,b1,b2,b3} */
  double ion_cmp[8];  
/** leap seconds (s) */
  int leaps;   
/** carrier wave lengths (m) */
  double lam[MAXSAT][NFREQ]; 
/** code bias (0:p1-p2,1:p1-c1,2:p2-c2) (m) */
  double cbias[MAXSAT][3]; 
/** wide-lane bias (cycle) */
  double wlbias[MAXSAT];  
/** glonass code-phase bias {1C,1P,2C,2P} (m) */
  double glo_cpbias[4];  
/** glonass frequency channel number + 8 */
  char glo_fcn[MAXPRNGLO+1]; 
/** satellite antenna pcv */
  pcv_t pcvs[MAXSAT]; 
/** SBAS satellite corrections */
  sbssat_t sbssat;  
/** SBAS ionosphere corrections */
  sbsion_t sbsion[MAXBAND+1]; 
/** DGPS corrections */
  dgps_t dgps[MAXSAT]; 
/** SSR corrections */
  ssr_t ssr[MAXSAT];  
/** LEX ephemeris */
  lexeph_t lexeph[MAXSAT]; 
/** LEX ionosphere correction */
  lexion_t lexion;  
} nav_t;

/** station parameter type */
typedef struct {    
/** marker name */
  char name [MAXANT]; 
/** marker number */
  char marker [MAXANT]; 
/** antenna descriptor */
  char antdes [MAXANT]; 
/** antenna serial number */
  char antsno [MAXANT]; 
/** receiver type descriptor */
  char rectype[MAXANT]; 
/** receiver firmware version */
  char recver [MAXANT]; 
/** receiver serial number */
  char recsno [MAXANT]; 
/** antenna setup id */
  int antsetup;   
/** ITRF realization year */
  int itrf;   
/** antenna delta type (0:enu,1:xyz) */
  int deltype;    
/** station position (ecef) (m) */
  double pos[3];   
/** antenna position delta (e/n/u or x/y/z) (m) */
  double del[3];   
/** antenna height (m) */
  double hgt;    
} sta_t;

/** solution type */
typedef struct {    
/** time (GPST) */
  gtime_t time;   
/** position/velocity (m|m/s) */
  double rr[6];   
/** {x,y,z,vx,vy,vz} or {e,n,u,ve,vn,vu} */
       
/** position variance/covariance (m^2) */
  float  qr[6];   
/** {c_xx,c_yy,c_zz,c_xy,c_yz,c_zx} or */
       
/** {c_ee,c_nn,c_uu,c_en,c_nu,c_ue} */
       
/** receiver clock bias to time systems (s) */
  double dtr[6];   
/** type (0:xyz-ecef,1:enu-baseline) */
  unsigned char type; 
/** solution status (SOLQ_???) */
  unsigned char stat; 
/** number of valid satellites */
  unsigned char ns; 
/** age of differential (s) */
  float age;   
/** AR ratio factor for valiation */
  float ratio;    
} sol_t;

/** solution buffer type */
typedef struct {    
/** number of solution/max number of buffer */
  int n,nmax;    
/** cyclic buffer flag */
  int cyclic;    
/** start/end index */
  int start,end;   
/** current solution time */
  gtime_t time;   
/** solution data */
  sol_t *data;    
/** reference position {x,y,z} (ecef) (m) */
  double rb[3];   
/** message buffer */
  unsigned char buff[MAXSOLMSG+1]; 
/** number of byte in message buffer */
  int nb;    
} solbuf_t;

/** solution status type */
typedef struct {    
/** time (GPST) */
  gtime_t time;   
/** satellite number */
  unsigned char sat;  
/** frequency (1:L1,2:L2,...) */
  unsigned char frq;  
/** azimuth/elevation angle (rad) */
  float az,el;    
/** pseudorange residual (m) */
  float resp;    
/** carrier-phase residual (m) */
  float resc;    
/** flags: (vsat<<5)+(slip<<3)+fix */
  unsigned char flag; 
/** signal strength (0.25 dBHz) */
  unsigned char snr;  
/** lock counter */
  unsigned short lock;  
/** outage counter */
  unsigned short outc;  
/** slip counter */
  unsigned short slipc; 
/** reject counter */
  unsigned short rejc;  
} solstat_t;

/** solution status buffer type */
typedef struct {    
/** number of solution/max number of buffer */
  int n,nmax;    
/** solution status data */
  solstat_t *data;  
} solstatbuf_t;

/** RTCM control struct type */
typedef struct {    
/** station id */
  int staid;   
/** station health */
  int stah;   
/** sequence number for rtcm 2 or iods msm */
  int seqno;   
/** output message type */
  int outtype;    
/** message time */
  gtime_t time;   
/** message start time */
  gtime_t time_s;  
/** observation data (uncorrected) */
  obs_t obs;   
/** satellite ephemerides */
  nav_t nav;   
/** station parameters */
  sta_t sta;   
/** output of dgps corrections */
  dgps_t *dgps;   
/** output of ssr corrections */
  ssr_t ssr[MAXSAT];  
/** special message */
  char msg[128];   
/** last message type */
  char msgtype[256];  
/** msm signal types */
  char msmtype[6][128]; 
/** obs data complete flag (1:ok,0:not complete) */
  int obsflag;    
/** update satellite of ephemeris */
  int ephsat;    
/** carrier-phase measurement */
  double cp[MAXSAT][NFREQ+NEXOBS]; 
/** lock time */
  unsigned char lock[MAXSAT][NFREQ+NEXOBS]; 
/** loss of lock count */
  unsigned char loss[MAXSAT][NFREQ+NEXOBS]; 
/** last lock time */
  gtime_t lltime[MAXSAT][NFREQ+NEXOBS]; 
/** number of bytes in message buffer */ 
  int nbyte;   
/** number of bits in word buffer */ 
  int nbit;   
/** message length (bytes) */
  int len;    
/** message buffer */
  unsigned char buff[1200]; 
/** word buffer for rtcm 2 */
  unsigned int word;  
/** message count of RTCM 2 (1-99:1-99,0:other) */
  unsigned int nmsg2[100]; 
/** message count of RTCM 3 (1-299:1001-1299,0:ohter) */
  unsigned int nmsg3[300]; 
/** RTCM dependent options */
  char opt[256];   
} rtcm_t;

/** rinex control struct type */
typedef struct {    
/** message time */
  gtime_t time;   
/** rinex version */
  double ver;    
/** rinex file type ('O','N',...) */
  char type;    
/** navigation system */
  int  sys;    
/** time system */
  int  tsys;    
/** rinex obs types */
  char tobs[6][MAXOBSTYPE][4]; 
/** observation data */
  obs_t  obs;    
/** navigation data */
  nav_t  nav;    
/** station info */
  sta_t  sta;    
/** ephemeris satellite number */
  int  ephsat;   
/** rinex dependent options */
  char opt[256];  
} rnxctr_t;

/** download url type */
typedef struct {    
/** data type */
  char type[32];   
/** url path */
  char path[1024];  
/** local directory */
  char dir [1024];  
/** time interval (s) */
  double tint;    
} url_t;

/** option type */
typedef struct opt_t {    
/** option name */
  char *name;    
/** option format (0:int,1:double,2:string,3:enum) */
  int format;    
/** pointer to option variable */
  void *var;   
/** option comment/enum labels/unit */
  char *comment;   
} opt_t;

/** extended receiver error model */
typedef struct {    
/** model enabled */
  int ena[4];    
/** code errors (m) */
  double cerr[4][NFREQ*2]; 
/** carrier-phase errors (m) */
  double perr[4][NFREQ*2]; 
/** gps-glonass h/w bias (m) */
  double gpsglob[NFREQ]; 
/** glonass interchannel bias (m/fn) */
  double gloicb [NFREQ]; 
} exterr_t;

/** SNR mask type */
typedef struct {    
/** enable flag {rover,base} */
  int ena[2];    
/** mask (dBHz) at 5,10,...85 deg */
  double mask[NFREQ][9]; 
} snrmask_t;

/** processing options type */
typedef struct {    
/** positioning mode (PMODE_???) */
  int mode;   
/** solution type (0:forward,1:backward,2:combined) */
  int soltype;    
/** number of frequencies (1:L1,2:L1+L2,3:L1+L2+L5) */
  int nf;    
/** navigation system */
  int navsys;    
/** elevation mask angle (rad) */
  double elmin;   
/** SNR mask */
  snrmask_t snrmask;  
/** satellite ephemeris/clock (EPHOPT_???) */
  int sateph;    
/** AR mode (0:off,1:continuous,2:instantaneous,3:fix and hold,4:ppp-ar) */
  int modear;    
/** GLONASS AR mode (0:off,1:on,2:auto cal,3:ext cal) */
  int glomodear;   
/** BeiDou AR mode (0:off,1:on) */
  int bdsmodear;   
/** obs outage count to reset bias */
  int maxout;    
/** min lock count to fix ambiguity */
  int minlock;    
/** min fix count to hold ambiguity */
  int minfix;    
/** ionosphere option (IONOOPT_???) */
  int ionoopt;    
/** troposphere option (TROPOPT_???) */
  int tropopt;    
/** dynamics model (0:none,1:velociy,2:accel) */
  int dynamics;   
/** earth tide correction (0:off,1:solid,2:solid+otl+pole) */
  int tidecorr;   
/** number of filter iteration */
  int niter;   
/** code smoothing window size (0:none) */
  int codesmooth;  
/** interpolate reference obs (for post mission) */
  int intpref;    
/** SBAS correction options */
  int sbascorr;   
/** SBAS satellite selection (0:all) */
  int sbassatsel;  
/** rover position for fixed mode */
  int rovpos;    
/** base position for relative mode */
  int refpos;    
/** (0:pos in prcopt,  1:average of single pos, */
       
/**  2:read from file, 3:rinex header, 4:rtcm pos) */
       
/** code/phase error ratio */
  double eratio[NFREQ]; 
/** measurement error factor */
  double err[5];   
/** [0]:reserved */
       
/** [1-3]:error factor a/b/c of phase (m) */
       
/** [4]:doppler frequency (hz) */
       
/** initial-state std [0]bias,[1]iono [2]trop */
  double std[3];   
/** process-noise std [0]bias,[1]iono [2]trop [3]acch [4]accv */
  double prn[5];   
/** satellite clock stability (sec/sec) */
  double sclkstab;  
/** AR validation threshold */
  double thresar[4];  
/** elevation mask of AR for rising satellite (deg) */
  double elmaskar;  
/** elevation mask to hold ambiguity (deg) */
  double elmaskhold;  
/** slip threshold of geometry-free phase (m) */
  double thresslip; 
/** max difference of time (sec) */
  double maxtdiff;  
/** reject threshold of innovation (m) */
  double maxinno;  
/** reject threshold of gdop */
  double maxgdop;  
/** baseline length constraint {const,sigma} (m) */
  double baseline[2]; 
/** rover position for fixed mode {x,y,z} (ecef) (m) */
  double ru[3];   
/** base position for relative mode {x,y,z} (ecef) (m) */
  double rb[3];   
/** antenna types {rover,base} */
  char anttype[2][MAXANT]; 
/** antenna delta {{rov_e,rov_n,rov_u},{ref_e,ref_n,ref_u}} */
  double antdel[2][3]; 
/** receiver antenna parameters {rov,base} */
  pcv_t pcvr[2];   
/** excluded satellites (1:excluded,2:included) */
  unsigned char exsats[MAXSAT]; 
/** rinex options {rover,base} */
  char rnxopt[2][256]; 
/** positioning options */
  int  posopt[6];  
/** solution sync mode (0:off,1:on) */
  int  syncsol;   
/** ocean tide loading parameters {rov,base} */
  double odisp[2][6*11]; 
/** extended receiver error model */
  exterr_t exterr;  
} prcopt_t;

/** solution options type */
typedef struct {    
/** solution format (SOLF_???) */
  int posf;   
/** time system (TIMES_???) */
  int times;   
/** time format (0:sssss.s,1:yyyy/mm/dd hh:mm:ss.s) */
  int timef;   
/** time digits under decimal point */
  int timeu;   
/** latitude/longitude format (0:ddd.ddd,1:ddd mm ss) */
  int degf;   
/** output header (0:no,1:yes) */
  int outhead;    
/** output processing options (0:no,1:yes) */
  int outopt;    
/** datum (0:WGS84,1:Tokyo) */
  int datum;   
/** height (0:ellipsoidal,1:geodetic) */
  int height;    
/** geoid model (0:EGM96,1:JGD2000) */
  int geoid;   
/** solution of static mode (0:all,1:single) */
  int solstatic;   
/** solution statistics level (0:off,1:states,2:residuals) */
  int sstat;   
/** debug trace level (0:off,1-5:debug) */
  int trace;   
/** nmea output interval (s) (<0:no,0:all) */
  double nmeaintv[2]; 
/** nmeaintv[0]:gprmc,gpgga,nmeaintv[1]:gpgsv */
       
/** field separator */
  char sep[64];   
/** program name */
  char prog[64];   
} solopt_t;

/** file options type */
typedef struct {    
/** satellite antenna parameters file */
  char satantp[MAXSTRPATH]; 
/** receiver antenna parameters file */
  char rcvantp[MAXSTRPATH]; 
/** station positions file */
  char stapos [MAXSTRPATH]; 
/** external geoid data file */
  char geoid  [MAXSTRPATH]; 
/** ionosphere data file */
  char iono [MAXSTRPATH]; 
/** dcb data file */
  char dcb  [MAXSTRPATH]; 
/** eop data file */
  char eop  [MAXSTRPATH]; 
/** ocean tide loading blq file */
  char blq  [MAXSTRPATH]; 
/** ftp/http temporaly directory */
  char tempdir[MAXSTRPATH]; 
/** google earth exec file */
  char geexe  [MAXSTRPATH]; 
/** solution statistics file */
  char solstat[MAXSTRPATH]; 
/** debug trace file */
  char trace  [MAXSTRPATH]; 
} filopt_t;

/** RINEX options type */
typedef struct {    
/** time start/end */
  gtime_t ts,te;   
/** time interval (s) */
  double tint;    
/** time unit for multiple-session (s) */
  double tunit;   
/** RINEX version */
  double rnxver;   
/** navigation system */
  int navsys;    
/** observation type */
  int obstype;    
/** frequency type */
  int freqtype;   
/** code mask {GPS,GLO,GAL,QZS,SBS,CMP} */
  char mask[6][64]; 
/** station id for rinex file name */
  char staid [32];  
/** program */
  char prog  [32];  
/** run-by */
  char runby [32];  
/** marker name */
  char marker[64];  
/** marker number */
  char markerno[32];  
/** marker type (ver.3) */
  char markertype[32]; 
/** observer/agency */
  char name[2][32]; 
/** receiver #/type/vers */
  char rec [3][32]; 
/** antenna #/type */
  char ant [3][32]; 
/** approx position x/y/z */
  double apppos[3]; 
/** antenna delta h/e/n */
  double antdel[3]; 
/** comments */
  char comment[MAXCOMMENT][64]; 
/** receiver dependent options */
  char rcvopt[256]; 
/** excluded satellites */
  unsigned char exsats[MAXSAT]; 
/** scan obs types */
  int scanobs;    
/** output iono correction */
  int outiono;    
/** output time system correction */
  int outtime;    
/** output leap seconds */
  int outleaps;   
/** auto approx position */
  int autopos;    
/** first obs time */
  gtime_t tstart;  
/** last obs time */
  gtime_t tend;   
/** approx log start time for rtcm */
  gtime_t trtcm;   
/** obs types {GPS,GLO,GAL,QZS,SBS,CMP} */
  char tobs[6][MAXOBSTYPE][4]; 
/** number of obs types {GPS,GLO,GAL,QZS,SBS,CMP} */
  int nobs[6];    
} rnxopt_t;

/** satellite status type */
typedef struct {    
/** navigation system */
  unsigned char sys;  
/** valid satellite flag single */
  unsigned char vs; 
/** azimuth/elevation angles {az,el} (rad) */
  double azel[2];  
/** residuals of pseudorange (m) */
  double resp[NFREQ]; 
/** residuals of carrier-phase (m) */
  double resc[NFREQ]; 
/** valid satellite flag */
  unsigned char vsat[NFREQ]; 
/** signal strength (0.25 dBHz) */
  unsigned char snr [NFREQ]; 
/** ambiguity fix flag (1:fix,2:float,3:hold) */
  unsigned char fix [NFREQ]; 
/** cycle-slip flag */
  unsigned char slip[NFREQ]; 
/** lock counter of phase */
  unsigned int lock [NFREQ]; 
/** obs outage counter of phase */
  unsigned int outc [NFREQ]; 
/** cycle-slip counter */
  unsigned int slipc[NFREQ]; 
/** reject counter */
  unsigned int rejc [NFREQ]; 
/** geometry-free phase L1-L2 (m) */
  double  gf;    
/** geometry-free phase L1-L5 (m) */
  double  gf2;    
/** phase windup (cycle) */
  double  phw;    
/** previous carrier-phase time */
  gtime_t pt[2][NFREQ]; 
/** previous carrier-phase observable (cycle) */
  double  ph[2][NFREQ]; 
} ssat_t;

/** ambiguity control type */
typedef struct {    
/** last epoch */
  gtime_t epoch[4]; 
/** fix counter */
  int fixcnt;    
/** fix flags */
  char flags[MAXSAT]; 
/** number of epochs */
  double n[4];    
/** linear combination average */
  double LC [4];   
/** linear combination variance */
  double LCv[4];   
} ambc_t;

/** RTK control/result type */
typedef struct {    
/** RTK solution */
  sol_t  sol;    
/** base position/velocity (ecef) (m|m/s) */
  double rb[6];   
/** number of float states/fixed states */
  int nx,na;   
/** time difference between current and previous (s) */
  double tt;   
/** float states and their covariance */
  double *x, *P;   
/** fixed states and their covariance */
  double *xa,*Pa;  
/** number of continuous fixes of ambiguity */
  int nfix;   
/** ambibuity control */
  ambc_t ambc[MAXSAT]; 
/** satellite status */
  ssat_t ssat[MAXSAT]; 
/** bytes in error message buffer */
  int neb;    
/** error message buffer */
  char errbuf[MAXERRMSG]; 
/** processing options */
  prcopt_t opt;   
} rtk_t;

/** receiver raw data control type */
typedef struct {    
/** message time */
  gtime_t time;   
/** observation data time */
  gtime_t tobs;   
/** observation data */
  obs_t obs;   
/** observation data buffer */
  obs_t obuf;    
/** satellite ephemerides */
  nav_t nav;   
/** station parameters */
  sta_t sta;   
/** sat number of update ephemeris (0:no satellite) */
  int ephsat;    
/** SBAS message */
  sbsmsg_t sbsmsg;  
/** last message type */
  char msgtype[256];  
/** subframe buffer */
  unsigned char subfrm[MAXSAT][380];  
/** LEX message */
  lexmsg_t lexmsg;  
/** lock time (s) */
  double lockt[MAXSAT][NFREQ+NEXOBS]; 
/** carrier params for ss2 */
  double icpp[MAXSAT],off[MAXSAT],icpc; 
/** L1/CA pseudrange/doppler for javad */
  double prCA[MAXSAT],dpCA[MAXSAT]; 
/** half-cycle add flag */
  unsigned char halfc[MAXSAT][NFREQ+NEXOBS]; 
/** frequency number for javad */
  char freqn[MAXOBS]; 
/** number of bytes in message buffer */ 
  int nbyte;   
/** message length (bytes) */
  int len;    
/** issue of data */
  int iod;    
/** time of day (ms) */
  int tod;    
/** time base (0:gpst,1:utc(usno),2:glonass,3:utc(su) */
  int tbase;   
/** general purpose flag */
  int flag;   
/** output message type */
  int outtype;    
/** message buffer */
  unsigned char buff[MAXRAWLEN]; 
/** receiver dependent options */
  char opt[256];   
/** RT17: Reiceve time of week for week rollover detection */
  double receive_time;
/** RT17: Total size of packet to be read */
  unsigned int plen;  
/** RT17: How many packet bytes have been read so far */
  unsigned int pbyte; 
/** RT17: Last page number */
  unsigned int page;  
/** RT17: Current reply number */
  unsigned int reply; 
/** RT17: week number */
  int week;   
/** RT17: Packet buffer */
  unsigned char pbuff[255+4+2]; 
} raw_t;

/** stream type */
typedef struct {    
/** type (STR_???) */
  int type;   
/** mode (STR_MODE_?) */
  int mode;   
/** state (-1:error,0:close,1:open) */
  int state;   
/** input bytes/rate */
  unsigned int inb,inr; 
/** output bytes/rate */
  unsigned int outb,outr; 
/** tick/active tick */
  unsigned int tick,tact; 
/** input/output bytes at tick */
  unsigned int inbt,outbt; 
/** lock flag */
  lock_t lock;    
/** type dependent port control struct */
  void *port;    
/** stream path */
  char path[MAXSTRPATH]; 
/** stream message */
  char msg [MAXSTRMSG];  
} stream_t;

/** stream converter type */
typedef struct {    
/** input and output stream type */
  int itype,otype;  
/** number of output messages */
  int nmsg;   
/** output message types */
  int msgs[32];   
/** output message intervals (s) */
  double tint[32];  
/** cycle tick of output message */
  unsigned int tick[32]; 
/** satellites of output ephemeris */
  int ephsat[32];  
/** station info selection (0:remote,1:local) */
  int stasel;    
/** rtcm input data buffer */
  rtcm_t rtcm;    
/** raw  input data buffer */
  raw_t raw;   
/** rtcm output data buffer */
  rtcm_t out;    
} strconv_t;

/** stream server type */
typedef struct {    
/** server state (0:stop,1:running) */
  int state;   
/** server cycle (ms) */
  int cycle;   
/** input/monitor buffer size (bytes) */
  int buffsize;   
/** NMEA request cycle (ms) (0:no) */
  int nmeacycle;   
/** number of streams (1 input + (nstr-1) outputs */
  int nstr;   
/** data length in peek buffer (bytes) */
  int npb;    
/** NMEA request position (ecef) (m) */
  double nmeapos[3];  
/** input buffers */
  unsigned char *buff; 
/** peek buffer */
  unsigned char *pbuf; 
/** start tick */
  unsigned int tick;  
/** input/output streams */
  stream_t stream[16]; 
/** stream converter */
  strconv_t *conv[16]; 
/** server thread */
  thread_t thread;  
/** lock flag */
  lock_t lock;    
} strsvr_t;

/** RTK server type */
typedef struct {    
/** server state (0:stop,1:running) */
  int state;   
/** processing cycle (ms) */
  int cycle;   
/** NMEA request cycle (ms) (0:no req) */
  int nmeacycle;   
/** NMEA request (0:no,1:nmeapos,2:single sol) */
  int nmeareq;    
/** NMEA request position (ecef) (m) */
  double nmeapos[3];  
/** input buffer size (bytes) */
  int buffsize;   
/** input format {rov,base,corr} */
  int format[3];   
/** output solution options {sol1,sol2} */
  solopt_t solopt[2]; 
/** ephemeris select (0:all,1:rover,2:base,3:corr) */
  int navsel;    
/** number of sbas message */
  int nsbs;   
/** number of solution buffer */
  int nsol;   
/** RTK control/result struct */
  rtk_t rtk;   
/** bytes in input buffers {rov,base} */
  int nb [3];    
/** bytes in soulution buffers */
  int nsb[2];    
/** bytes in input peek buffers */
  int npb[3];    
/** input buffers {rov,base,corr} */
  unsigned char *buff[3]; 
/** output buffers {sol1,sol2} */
  unsigned char *sbuf[2]; 
/** peek buffers {rov,base,corr} */
  unsigned char *pbuf[3]; 
/** solution buffer */
  sol_t solbuf[MAXSOLBUF]; 
/** input message counts */
  unsigned int nmsg[3][10]; 
/** receiver raw control {rov,base,corr} */
  raw_t  raw [3];  
/** RTCM control {rov,base,corr} */
  rtcm_t rtcm[3];  
/** download time {rov,base,corr} */
  gtime_t ftime[3]; 
/** download paths {rov,base,corr} */
  char files[3][MAXSTRPATH]; 
/** observation data {rov,base,corr} */
  obs_t obs[3][MAXOBSBUF]; 
/** navigation data */
  nav_t nav;   
/** SBAS message buffer */
  sbsmsg_t sbsmsg[MAXSBSMSG]; 
/** streams {rov,base,corr,sol1,sol2,logr,logb,logc} */
  stream_t stream[8]; 
/** monitor stream */
  stream_t *moni;  
/** start tick */
  unsigned int tick;  
/** server thread */
  thread_t thread;  
/** CPU time (ms) for a processing cycle */
  int cputime;    
/** missing observation data count */
  int prcout;    
/** lock flag */
  lock_t lock;    
} rtksvr_t;
/** @}
* @defgroup  RTKLIB_global global variables 
* @{
*/
/** chi-sqr(n) table (alpha=0.001) */
DLL_EXPORT const double chisqr[];
/** carrier wave length (m) {L1,L2,...} */
DLL_EXPORT const double lam_carr[];
/** default positioning options */
DLL_EXPORT const prcopt_t prcopt_default;
/** default solution output options */
DLL_EXPORT const solopt_t solopt_default;
/** SBAS IGP band 0-8 */
DLL_EXPORT const sbsigpband_t igpband1[][8];
/** SBAS IGP band 9-10 */
DLL_EXPORT const sbsigpband_t igpband2[][5];
/** stream format strings */
DLL_EXPORT const char *formatstrs[];
/** system options table */
DLL_EXPORT opt_t sysopts[];
/** @}
* @defgroup  RTKLIB_satellites satellites, systems, codes satellites, systems, codes functions
* @{
*/
DLL_EXPORT int  satno (int sys, int prn);
DLL_EXPORT int  satsys  (int sat, int *prn);
DLL_EXPORT int  satid2no(const char *id);
DLL_EXPORT void satno2id(int sat, char *id);
DLL_EXPORT unsigned char obs2code(const char *obs, int *freq);
DLL_EXPORT char *code2obs(unsigned char code, int *freq);
DLL_EXPORT int  satexclude(int sat, int svh, const prcopt_t *opt);
DLL_EXPORT int  testsnr(int base, int freq, double el, double snr,
      const snrmask_t *mask);
DLL_EXPORT void setcodepri(int sys, int freq, const char *pri);
DLL_EXPORT int  getcodepri(int sys, unsigned char code, const char *opt);
/** @}
* @defgroup  RTKLIB_matrix matrix and vector functions
* @{
*/
DLL_EXPORT double *mat  (int n, int m);
DLL_EXPORT int  *imat (int n, int m);
DLL_EXPORT double *zeros(int n, int m);
DLL_EXPORT double *eye  (int n);
DLL_EXPORT double dot (const double *a, const double *b, int n);
DLL_EXPORT double norm(const double *a, int n);
DLL_EXPORT void cross3(const double *a, const double *b, double *c);
DLL_EXPORT int  normv3(const double *a, double *b);
DLL_EXPORT void matcpy(double *A, const double *B, int n, int m);
DLL_EXPORT void matmul(const char *tr, int n, int k, int m, double alpha,
     const double *A, const double *B, double beta, double *C);
DLL_EXPORT int  matinv(double *A, int n);
DLL_EXPORT int  solve (const char *tr, const double *A, const double *Y, int n,
     int m, double *X);
DLL_EXPORT int  lsq (const double *A, const double *y, int n, int m, double *x,
     double *Q);
DLL_EXPORT int  filter(double *x, double *P, const double *H, const double *v,
     const double *R, int n, int m);
DLL_EXPORT int  smoother(const double *xf, const double *Qf, const double *xb,
      const double *Qb, int n, double *xs, double *Qs);
DLL_EXPORT void matprint (const double *A, int n, int m, int p, int q);
DLL_EXPORT void matfprint(const double *A, int n, int m, int p, int q, FILE *fp);
/** @}
* @defgroup  RTKLIB_time time and string functions
* @{
*/
DLL_EXPORT double  str2num(const char *s, int i, int n);
DLL_EXPORT int  str2time(const char *s, int i, int n, gtime_t *t);
DLL_EXPORT void  time2str(gtime_t t, char *str, int n);
DLL_EXPORT gtime_t epoch2time(const double *ep);
DLL_EXPORT void  time2epoch(gtime_t t, double *ep);
DLL_EXPORT gtime_t gpst2time(int week, double sec);
DLL_EXPORT double  time2gpst(gtime_t t, int *week);
DLL_EXPORT gtime_t gst2time(int week, double sec);
DLL_EXPORT double  time2gst(gtime_t t, int *week);
DLL_EXPORT gtime_t bdt2time(int week, double sec);
DLL_EXPORT double  time2bdt(gtime_t t, int *week);
DLL_EXPORT char  *time_str(gtime_t t, int n);

DLL_EXPORT gtime_t timeadd  (gtime_t t, double sec);
DLL_EXPORT double  timediff (gtime_t t1, gtime_t t2);
DLL_EXPORT gtime_t gpst2utc (gtime_t t);
DLL_EXPORT gtime_t utc2gpst (gtime_t t);
DLL_EXPORT gtime_t gpst2bdt (gtime_t t);
DLL_EXPORT gtime_t bdt2gpst (gtime_t t);
DLL_EXPORT gtime_t timeget  (void);
DLL_EXPORT void  timeset  (gtime_t t);
DLL_EXPORT double  time2doy (gtime_t t);
DLL_EXPORT double  utc2gmst (gtime_t t, double ut1_utc);
DLL_EXPORT int read_leaps(const char *file);

DLL_EXPORT int adjgpsweek(int week);
DLL_EXPORT unsigned int tickget(void);
DLL_EXPORT void sleepms(int ms);

DLL_EXPORT int reppath(const char *path, char *rpath, gtime_t time, const char *rov,
     const char *base);
DLL_EXPORT int reppaths(const char *path, char *rpaths[], int nmax, gtime_t ts,
      gtime_t te, const char *rov, const char *base);
/** @}
* @defgroup  RTKLIB_coordinates coordinates transformation 
* @{
*/
DLL_EXPORT void ecef2pos(const double *r, double *pos);
DLL_EXPORT void pos2ecef(const double *pos, double *r);
DLL_EXPORT void ecef2enu(const double *pos, const double *r, double *e);
DLL_EXPORT void enu2ecef(const double *pos, const double *e, double *r);
DLL_EXPORT void covenu  (const double *pos, const double *P, double *Q);
DLL_EXPORT void covecef (const double *pos, const double *Q, double *P);
DLL_EXPORT void xyz2enu (const double *pos, double *E);
DLL_EXPORT void eci2ecef(gtime_t tutc, const double *erpv, double *U, double *gmst);
DLL_EXPORT void deg2dms (double deg, double *dms);
DLL_EXPORT double dms2deg(const double *dms);
/** @}
* @defgroup  RTKLIB_input input and output functions
* @{
*/
DLL_EXPORT void readpos(const char *file, const char *rcv, double *pos);
DLL_EXPORT int  sortobs(obs_t *obs);
DLL_EXPORT void uniqnav(nav_t *nav);
DLL_EXPORT int  screent(gtime_t time, gtime_t ts, gtime_t te, double tint);
DLL_EXPORT int  readnav(const char *file, nav_t *nav);
DLL_EXPORT int  savenav(const char *file, const nav_t *nav);
DLL_EXPORT void freeobs(obs_t *obs);
DLL_EXPORT void freenav(nav_t *nav, int opt);
DLL_EXPORT int  readblq(const char *file, const char *sta, double *odisp);
DLL_EXPORT int  readerp(const char *file, erp_t *erp);
DLL_EXPORT int  geterp (const erp_t *erp, gtime_t time, double *val);
/** @}
* @defgroup  RTKLIB_debug debug trace functions
* @{
*/
DLL_EXPORT void traceopen(const char *file);
DLL_EXPORT void traceclose(void);
DLL_EXPORT void tracelevel(int level);
DLL_EXPORT void trace  (int level, const char *format, ...);
DLL_EXPORT void tracet (int level, const char *format, ...);
DLL_EXPORT void tracemat (int level, const double *A, int n, int m, int p, int q);
DLL_EXPORT void traceobs (int level, const obsd_t *obs, int n);
DLL_EXPORT void tracenav (int level, const nav_t *nav);
DLL_EXPORT void tracegnav(int level, const nav_t *nav);
DLL_EXPORT void tracehnav(int level, const nav_t *nav);
DLL_EXPORT void tracepeph(int level, const nav_t *nav);
DLL_EXPORT void tracepclk(int level, const nav_t *nav);
DLL_EXPORT void traceb (int level, const unsigned char *p, int n);
/** @}
* @defgroup  RTKLIB_platform platform dependent functions
* @{
*/
DLL_EXPORT int execcmd(const char *cmd);
DLL_EXPORT int expath (const char *path, char *paths[], int nmax);
DLL_EXPORT void createdir(const char *path);
/** @}
* @defgroup  RTKLIB_positioning positioning models 
* @{
*/
DLL_EXPORT double satwavelen(int sat, int frq, const nav_t *nav);
DLL_EXPORT double satazel(const double *pos, const double *e, double *azel);
DLL_EXPORT double geodist(const double *rs, const double *rr, double *e);
DLL_EXPORT void dops(int ns, const double *azel, double elmin, double *dop);
DLL_EXPORT void csmooth(obs_t *obs, int ns);
/** @}
* @defgroup  RTKLIB_atmosphere atmosphere models 
* @{
*/
DLL_EXPORT double ionmodel(gtime_t t, const double *ion, const double *pos,
      const double *azel);
DLL_EXPORT double ionmapf(const double *pos, const double *azel);
DLL_EXPORT double ionppp(const double *pos, const double *azel, double re,
      double hion, double *pppos);
DLL_EXPORT double tropmodel(gtime_t time, const double *pos, const double *azel,
       double humi);
DLL_EXPORT double tropmapf(gtime_t time, const double *pos, const double *azel,
      double *mapfw);
DLL_EXPORT int iontec(gtime_t time, const nav_t *nav, const double *pos,
     const double *azel, int opt, double *delay, double *var);
DLL_EXPORT void readtec(const char *file, nav_t *nav, int opt);
DLL_EXPORT int ionocorr(gtime_t time, const nav_t *nav, int sat, const double *pos,
      const double *azel, int ionoopt, double *ion, double *var);
DLL_EXPORT int tropcorr(gtime_t time, const nav_t *nav, const double *pos,
      const double *azel, int tropopt, double *trp, double *var);
DLL_EXPORT void stec_read(const char *file, nav_t *nav);
DLL_EXPORT int stec_grid(const nav_t *nav, const double *pos, int nmax, int *index,
      double *dist);
DLL_EXPORT int stec_data(stec_t *stec, gtime_t time, int sat, double *iono,
      double *rate, double *rms, int *slip);
DLL_EXPORT int stec_ion(gtime_t time, const nav_t *nav, int sat, const double *pos,
      const double *azel, double *iono, double *rate, double *var,
      int *brk);
DLL_EXPORT void stec_free(nav_t *nav);
/** @}
* @defgroup  RTKLIB_antenna antenna models 
* @{
*/
DLL_EXPORT int  readpcv(const char *file, pcvs_t *pcvs);
DLL_EXPORT pcv_t *searchpcv(int sat, const char *type, gtime_t time,
       const pcvs_t *pcvs);
DLL_EXPORT void antmodel(const pcv_t *pcv, const double *del, const double *azel,
      int opt, double *dant);
DLL_EXPORT void antmodel_s(const pcv_t *pcv, double nadir, double *dant);
/** @}
* @defgroup  RTKLIB_earth earth tide models 
* @{
*/
DLL_EXPORT void sunmoonpos(gtime_t tutc, const double *erpv, double *rsun,
      double *rmoon, double *gmst);
DLL_EXPORT void tidedisp(gtime_t tutc, const double *rr, int opt, const erp_t *erp,
      const double *odisp, double *dr);
/** @}
* @defgroup  RTKLIB_geiod geiod models 
* @{
*/
DLL_EXPORT int opengeoid(int model, const char *file);
DLL_EXPORT void closegeoid(void);
DLL_EXPORT double geoidh(const double *pos);
/** @}
* @defgroup  RTKLIB_datum datum transformation 
* @{
*/
DLL_EXPORT int loaddatump(const char *file);
DLL_EXPORT int tokyo2jgd(double *pos);
DLL_EXPORT int jgd2tokyo(double *pos);
/** @}
* @defgroup  RTKLIB_rinex rinex functions
* @{
*/
DLL_EXPORT int readrnx (const char *file, int rcv, const char *opt, obs_t *obs,
      nav_t *nav, sta_t *sta);
DLL_EXPORT int readrnxt(const char *file, int rcv, gtime_t ts, gtime_t te,
      double tint, const char *opt, obs_t *obs, nav_t *nav,
      sta_t *sta);
DLL_EXPORT int readrnxc(const char *file, nav_t *nav);
DLL_EXPORT int outrnxobsh(FILE *fp, const rnxopt_t *opt, const nav_t *nav);
DLL_EXPORT int outrnxobsb(FILE *fp, const rnxopt_t *opt, const obsd_t *obs, int n,
      int epflag);
DLL_EXPORT int outrnxnavh (FILE *fp, const rnxopt_t *opt, const nav_t *nav);
DLL_EXPORT int outrnxgnavh(FILE *fp, const rnxopt_t *opt, const nav_t *nav);
DLL_EXPORT int outrnxhnavh(FILE *fp, const rnxopt_t *opt, const nav_t *nav);
DLL_EXPORT int outrnxlnavh(FILE *fp, const rnxopt_t *opt, const nav_t *nav);
DLL_EXPORT int outrnxqnavh(FILE *fp, const rnxopt_t *opt, const nav_t *nav);
DLL_EXPORT int outrnxcnavh(FILE *fp, const rnxopt_t *opt, const nav_t *nav);
DLL_EXPORT int outrnxnavb (FILE *fp, const rnxopt_t *opt, const eph_t *eph);
DLL_EXPORT int outrnxgnavb(FILE *fp, const rnxopt_t *opt, const geph_t *geph);
DLL_EXPORT int outrnxhnavb(FILE *fp, const rnxopt_t *opt, const seph_t *seph);
DLL_EXPORT int uncompress(const char *file, char *uncfile);
DLL_EXPORT int convrnx(int format, rnxopt_t *opt, const char *file, char **ofile);
DLL_EXPORT int  init_rnxctr (rnxctr_t *rnx);
DLL_EXPORT void free_rnxctr (rnxctr_t *rnx);
DLL_EXPORT int  open_rnxctr (rnxctr_t *rnx, FILE *fp);
DLL_EXPORT int  input_rnxctr(rnxctr_t *rnx, FILE *fp);
/** @}
* @defgroup  RTKLIB_ephemeris ephemeris and clock functions
* @{
*/
DLL_EXPORT double eph2clk (gtime_t time, const eph_t  *eph);
DLL_EXPORT double geph2clk(gtime_t time, const geph_t *geph);
DLL_EXPORT double seph2clk(gtime_t time, const seph_t *seph);
DLL_EXPORT void eph2pos (gtime_t time, const eph_t  *eph,  double *rs, double *dts,
      double *var);
DLL_EXPORT void geph2pos(gtime_t time, const geph_t *geph, double *rs, double *dts,
      double *var);
DLL_EXPORT void seph2pos(gtime_t time, const seph_t *seph, double *rs, double *dts,
      double *var);
DLL_EXPORT int  peph2pos(gtime_t time, int sat, const nav_t *nav, int opt,
      double *rs, double *dts, double *var);
DLL_EXPORT void satantoff(gtime_t time, const double *rs, int sat, const nav_t *nav,
      double *dant);
DLL_EXPORT int  satpos(gtime_t time, gtime_t teph, int sat, int ephopt,
     const nav_t *nav, double *rs, double *dts, double *var,
     int *svh);
DLL_EXPORT void satposs(gtime_t time, const obsd_t *obs, int n, const nav_t *nav,
      int sateph, double *rs, double *dts, double *var, int *svh);
DLL_EXPORT void readsp3(const char *file, nav_t *nav, int opt);
DLL_EXPORT int  readsap(const char *file, gtime_t time, nav_t *nav);
DLL_EXPORT int  readdcb(const char *file, nav_t *nav);
DLL_EXPORT void alm2pos(gtime_t time, const alm_t *alm, double *rs, double *dts);

DLL_EXPORT int tle_read(const char *file, tle_t *tle);
DLL_EXPORT int tle_name_read(const char *file, tle_t *tle);
DLL_EXPORT int tle_pos(gtime_t time, const char *name, const char *satno,
     const char *desig, const tle_t *tle, const erp_t *erp,
     double *rs);
/** @}
* @defgroup  RTKLIB_receiver receiver raw data functions
* @{
*/
DLL_EXPORT unsigned int getbitu(const unsigned char *buff, int pos, int len);
DLL_EXPORT int   getbits(const unsigned char *buff, int pos, int len);
DLL_EXPORT void setbitu(unsigned char *buff, int pos, int len, unsigned int data);
DLL_EXPORT void setbits(unsigned char *buff, int pos, int len, int data);
DLL_EXPORT unsigned int crc32  (const unsigned char *buff, int len);
DLL_EXPORT unsigned int crc24q (const unsigned char *buff, int len);
DLL_EXPORT unsigned short crc16(const unsigned char *buff, int len);
DLL_EXPORT int decode_word (unsigned int word, unsigned char *data);
DLL_EXPORT int decode_frame(const unsigned char *buff, eph_t *eph, alm_t *alm,
       double *ion, double *utc, int *leaps);
DLL_EXPORT int test_glostr(const unsigned char *buff);
DLL_EXPORT int decode_glostr(const unsigned char *buff, geph_t *geph);
DLL_EXPORT int decode_bds_d1(const unsigned char *buff, eph_t *eph);
DLL_EXPORT int decode_bds_d2(const unsigned char *buff, eph_t *eph);

DLL_EXPORT int init_raw (raw_t *raw);
DLL_EXPORT void free_raw  (raw_t *raw);
DLL_EXPORT int input_raw  (raw_t *raw, int format, unsigned char data);
DLL_EXPORT int input_rawf (raw_t *raw, int format, FILE *fp);

DLL_EXPORT int input_oem4  (raw_t *raw, unsigned char data);
DLL_EXPORT int input_oem3  (raw_t *raw, unsigned char data);
DLL_EXPORT int input_ubx (raw_t *raw, unsigned char data);
DLL_EXPORT int input_ss2 (raw_t *raw, unsigned char data);
DLL_EXPORT int input_cres  (raw_t *raw, unsigned char data);
DLL_EXPORT int input_stq (raw_t *raw, unsigned char data);
DLL_EXPORT int input_gw10  (raw_t *raw, unsigned char data);
DLL_EXPORT int input_javad (raw_t *raw, unsigned char data);
DLL_EXPORT int input_nvs (raw_t *raw, unsigned char data);
DLL_EXPORT int input_bnx (raw_t *raw, unsigned char data);
DLL_EXPORT int input_rt17  (raw_t *raw, unsigned char data);
DLL_EXPORT int input_lexr  (raw_t *raw, unsigned char data);
DLL_EXPORT int input_sirf  (raw_t *raw, unsigned char data);
DLL_EXPORT int input_oem4f (raw_t *raw, FILE *fp);
DLL_EXPORT int input_oem3f (raw_t *raw, FILE *fp);
DLL_EXPORT int input_ubxf  (raw_t *raw, FILE *fp);
DLL_EXPORT int input_ss2f  (raw_t *raw, FILE *fp);
DLL_EXPORT int input_cresf (raw_t *raw, FILE *fp);
DLL_EXPORT int input_stqf  (raw_t *raw, FILE *fp);
DLL_EXPORT int input_gw10f (raw_t *raw, FILE *fp);
DLL_EXPORT int input_javadf(raw_t *raw, FILE *fp);
DLL_EXPORT int input_nvsf  (raw_t *raw, FILE *fp);
DLL_EXPORT int input_bnxf  (raw_t *raw, FILE *fp);
DLL_EXPORT int input_rt17f (raw_t *raw, FILE *fp);
DLL_EXPORT int input_lexrf (raw_t *raw, FILE *fp);
DLL_EXPORT int input_sirff (raw_t *raw, FILE *fp);

DLL_EXPORT int gen_ubx (const char *msg, unsigned char *buff);
DLL_EXPORT int gen_stq (const char *msg, unsigned char *buff);
DLL_EXPORT int gen_nvs (const char *msg, unsigned char *buff);
DLL_EXPORT int gen_lexr(const char *msg, unsigned char *buff);
/** @}
* @defgroup  RTKLIB_rtcm rtcm functions
* @{
*/
DLL_EXPORT int init_rtcm (rtcm_t *rtcm);
DLL_EXPORT void free_rtcm  (rtcm_t *rtcm);
DLL_EXPORT int input_rtcm2 (rtcm_t *rtcm, unsigned char data);
DLL_EXPORT int input_rtcm3 (rtcm_t *rtcm, unsigned char data);
DLL_EXPORT int input_rtcm2f(rtcm_t *rtcm, FILE *fp);
DLL_EXPORT int input_rtcm3f(rtcm_t *rtcm, FILE *fp);
DLL_EXPORT int gen_rtcm2 (rtcm_t *rtcm, int type, int sync);
DLL_EXPORT int gen_rtcm3 (rtcm_t *rtcm, int type, int sync);

/** @}
* @defgroup RTKLIB_solution solution functions
* @{
*/
DLL_EXPORT void initsolbuf(solbuf_t *solbuf, int cyclic, int nmax);
DLL_EXPORT void freesolbuf(solbuf_t *solbuf);
DLL_EXPORT void freesolstatbuf(solstatbuf_t *solstatbuf);
DLL_EXPORT sol_t *getsol(solbuf_t *solbuf, int index);
DLL_EXPORT int addsol(solbuf_t *solbuf, const sol_t *sol);
DLL_EXPORT int readsol (char *files[], int nfile, solbuf_t *sol);
DLL_EXPORT int readsolt(char *files[], int nfile, gtime_t ts, gtime_t te,
      double tint, int qflag, solbuf_t *sol);
DLL_EXPORT int readsolstat(char *files[], int nfile, solstatbuf_t *statbuf);
DLL_EXPORT int readsolstatt(char *files[], int nfile, gtime_t ts, gtime_t te,
       double tint, solstatbuf_t *statbuf);
DLL_EXPORT int inputsol(unsigned char data, gtime_t ts, gtime_t te, double tint,
      int qflag, const solopt_t *opt, solbuf_t *solbuf);

DLL_EXPORT int outprcopts(unsigned char *buff, const prcopt_t *opt);
DLL_EXPORT int outsolheads(unsigned char *buff, const solopt_t *opt);
DLL_EXPORT int outsols  (unsigned char *buff, const sol_t *sol, const double *rb,
      const solopt_t *opt);
DLL_EXPORT int outsolexs(unsigned char *buff, const sol_t *sol, const ssat_t *ssat,
      const solopt_t *opt);
DLL_EXPORT void outprcopt(FILE *fp, const prcopt_t *opt);
DLL_EXPORT void outsolhead(FILE *fp, const solopt_t *opt);
DLL_EXPORT void outsol  (FILE *fp, const sol_t *sol, const double *rb,
      const solopt_t *opt);
DLL_EXPORT void outsolex(FILE *fp, const sol_t *sol, const ssat_t *ssat,
      const solopt_t *opt);
DLL_EXPORT int outnmea_rmc(unsigned char *buff, const sol_t *sol);
DLL_EXPORT int outnmea_gga(unsigned char *buff, const sol_t *sol);
DLL_EXPORT int outnmea_gsa(unsigned char *buff, const sol_t *sol,
      const ssat_t *ssat);
DLL_EXPORT int outnmea_gsv(unsigned char *buff, const sol_t *sol,
      const ssat_t *ssat);
/** @}
* @defgroup  RTKLIB_google google earth kml converter 
* @{
*/
DLL_EXPORT int convkml(const char *infile, const char *outfile, gtime_t ts,
     gtime_t te, double tint, int qflg, double *offset,
     int tcolor, int pcolor, int outalt, int outtime);
/** @}
* @defgroup  RTKLIB_sbas sbas functions
* @{
*/
DLL_EXPORT int  sbsreadmsg (const char *file, int sel, sbs_t *sbs);
DLL_EXPORT int  sbsreadmsgt(const char *file, int sel, gtime_t ts, gtime_t te,
       sbs_t *sbs);
DLL_EXPORT void sbsoutmsg(FILE *fp, sbsmsg_t *sbsmsg);
DLL_EXPORT int  sbsdecodemsg(gtime_t time, int prn, const unsigned int *words,
       sbsmsg_t *sbsmsg);
DLL_EXPORT int sbsupdatecorr(const sbsmsg_t *msg, nav_t *nav);
DLL_EXPORT int sbssatcorr(gtime_t time, int sat, const nav_t *nav, double *rs,
      double *dts, double *var);
DLL_EXPORT int sbsioncorr(gtime_t time, const nav_t *nav, const double *pos,
      const double *azel, double *delay, double *var);
DLL_EXPORT double sbstropcorr(gtime_t time, const double *pos, const double *azel,
       double *var);
/** @}
* @defgroup  RTKLIB_options options functions
* @{
*/
DLL_EXPORT opt_t *searchopt(const char *name, const opt_t *opts);
DLL_EXPORT int str2opt(opt_t *opt, const char *str);
DLL_EXPORT int opt2str(const opt_t *opt, char *str);
DLL_EXPORT int opt2buf(const opt_t *opt, char *buff);
DLL_EXPORT int loadopts(const char *file, opt_t *opts);
DLL_EXPORT int saveopts(const char *file, const char *mode, const char *comment,
      const opt_t *opts);
DLL_EXPORT void resetsysopts(void);
DLL_EXPORT void getsysopts(prcopt_t *popt, solopt_t *sopt, filopt_t *fopt);
DLL_EXPORT void setsysopts(const prcopt_t *popt, const solopt_t *sopt,
      const filopt_t *fopt);

/** @}
* @defgroup RTKLIB_stream data input and output functions
* @{
*/
DLL_EXPORT void strinitcom(void);
DLL_EXPORT void strinit  (stream_t *stream);
DLL_EXPORT void strlock  (stream_t *stream);
DLL_EXPORT void strunlock(stream_t *stream);
DLL_EXPORT int  stropen  (stream_t *stream, int type, int mode, const char *path);
DLL_EXPORT void strclose (stream_t *stream);
DLL_EXPORT int  strread  (stream_t *stream, unsigned char *buff, int n);
DLL_EXPORT int  strwrite (stream_t *stream, unsigned char *buff, int n);
DLL_EXPORT void strsync  (stream_t *stream1, stream_t *stream2);
DLL_EXPORT int  strstat  (stream_t *stream, char *msg);
DLL_EXPORT void strsum (stream_t *stream, int *inb, int *inr, int *outb, int *outr);
DLL_EXPORT void strsetopt(const int *opt);
DLL_EXPORT gtime_t strgettime(stream_t *stream);
DLL_EXPORT void strsendnmea(stream_t *stream, const double *pos);
DLL_EXPORT void strsendcmd(stream_t *stream, const char *cmd);
DLL_EXPORT void strsettimeout(stream_t *stream, int toinact, int tirecon);
DLL_EXPORT void strsetdir(const char *dir);
DLL_EXPORT void strsetproxy(const char *addr);
/** @}
* @defgroup  RTKLIB_integer integer ambiguity resolution 
* @{
*/
DLL_EXPORT int lambda(int n, int m, const double *a, const double *Q, double *F,
     double *s);
/** @}
* @defgroup  RTKLIB_standard standard positioning 
* @{
*/
DLL_EXPORT int pntpos(const obsd_t *obs, int n, const nav_t *nav,
     const prcopt_t *opt, sol_t *sol, double *azel,
     ssat_t *ssat, char *msg);
/** @}
* @defgroup  RTKLIB_precise precise positioning 
* @{
*/
DLL_EXPORT void rtkinit(rtk_t *rtk, const prcopt_t *opt);
DLL_EXPORT void rtkfree(rtk_t *rtk);
DLL_EXPORT int  rtkpos (rtk_t *rtk, const obsd_t *obs, int nobs, const nav_t *nav);
DLL_EXPORT int  rtkopenstat(const char *file, int level);
DLL_EXPORT void rtkclosestat(void);
/** @}
* @defgroup  RTKLIB_precise precise point positioning 
* @{
*/
DLL_EXPORT void pppos(rtk_t *rtk, const obsd_t *obs, int n, const nav_t *nav);
DLL_EXPORT int pppamb(rtk_t *rtk, const obsd_t *obs, int n, const nav_t *nav,
     const double *azel);
DLL_EXPORT int pppnx(const prcopt_t *opt);
DLL_EXPORT void pppoutsolstat(rtk_t *rtk, int level, FILE *fp);
DLL_EXPORT void windupcorr(gtime_t time, const double *rs, const double *rr,
      double *phw);
/** @}
* @defgroup  RTKLIB_post-processing post-processing positioning 
* @{
*/
DLL_EXPORT int postpos(gtime_t ts, gtime_t te, double ti, double tu,
     const prcopt_t *popt, const solopt_t *sopt,
     const filopt_t *fopt, char **infile, int n, char *outfile,
     const char *rov, const char *base);
/** @}
* @defgroup  RTKLIB_stream stream server functions
* @{
*/
DLL_EXPORT void strsvrinit (strsvr_t *svr, int nout);
DLL_EXPORT int  strsvrstart(strsvr_t *svr, int *opts, int *strs, char **paths,
       strconv_t **conv, const char *cmd,
       const double *nmeapos);
DLL_EXPORT void strsvrstop (strsvr_t *svr, const char *cmd);
DLL_EXPORT void strsvrstat (strsvr_t *svr, int *stat, int *byte, int *bps, char *msg);
DLL_EXPORT strconv_t *strconvnew(int itype, int otype, const char *msgs, int staid,
        int stasel, const char *opt);
DLL_EXPORT void strconvfree(strconv_t *conv);
/** @}
* @defgroup  RTKLIB_rtk rtk server functions
* @{
*/
DLL_EXPORT int  rtksvrinit  (rtksvr_t *svr);
DLL_EXPORT void rtksvrfree  (rtksvr_t *svr);
DLL_EXPORT int  rtksvrstart (rtksvr_t *svr, int cycle, int buffsize, int *strs,
       char **paths, int *formats, int navsel, char **cmds,
       char **rcvopts, int nmeacycle, int nmeareq,
       const double *nmeapos, prcopt_t *prcopt,
       solopt_t *solopt, stream_t *moni);
DLL_EXPORT void rtksvrstop  (rtksvr_t *svr, char **cmds);
DLL_EXPORT int  rtksvropenstr(rtksvr_t *svr, int index, int str, const char *path,
       const solopt_t *solopt);
DLL_EXPORT void rtksvrclosestr(rtksvr_t *svr, int index);
DLL_EXPORT void rtksvrlock  (rtksvr_t *svr);
DLL_EXPORT void rtksvrunlock(rtksvr_t *svr);
DLL_EXPORT int  rtksvrostat (rtksvr_t *svr, int type, gtime_t *time, int *sat,
       double *az, double *el, int **snr, int *vsat);
DLL_EXPORT void rtksvrsstat (rtksvr_t *svr, int *sstat, char *msg);
/** @}
* @defgroup  RTKLIB_downloader downloader functions
* @{
*/
DLL_EXPORT int dl_readurls(const char *file, char **types, int ntype, url_t *urls,
      int nmax);
DLL_EXPORT int dl_readstas(const char *file, char **stas, int nmax);
DLL_EXPORT int dl_exec(gtime_t ts, gtime_t te, double ti, int seqnos, int seqnoe,
     const url_t *urls, int nurl, char **stas, int nsta,
     const char *dir, const char *usr, const char *pwd,
     const char *proxy, int opts, char *msg, FILE *fp);
DLL_EXPORT void dl_test(gtime_t ts, gtime_t te, double ti, const url_t *urls,
      int nurl, char **stas, int nsta, const char *dir,
      int ncol, int datefmt, FILE *fp);
/** @}
* @defgroup  RTKLIB_application application defined functions
* @{
*/
DLL_EXPORT int showmsg(char *format,...);
DLL_EXPORT void settspan(gtime_t ts, gtime_t te);
DLL_EXPORT void settime(gtime_t time);
/** @}
* @defgroup  RTKLIB_qzss qzss lex functions
* @{
*/
DLL_EXPORT int lexupdatecorr(const lexmsg_t *msg, nav_t *nav, gtime_t *tof);
DLL_EXPORT int lexreadmsg(const char *file, int sel, lex_t *lex);
DLL_EXPORT void lexoutmsg(FILE *fp, const lexmsg_t *msg);
DLL_EXPORT int lexconvbin(int type, int format, const char *infile,
      const char *outfile);
DLL_EXPORT int lexeph2pos(gtime_t time, int sat, const nav_t *nav, double *rs,
      double *dts, double *var);
DLL_EXPORT int lexioncorr(gtime_t time, const nav_t *nav, const double *pos,
      const double *azel, double *delay, double *var);

/** @} */
/** @} */
#ifdef __cplusplus
}
#endif
/** RTKLIB_H */
#endif 
